/* ----------------------------------------------------------------------------
 *         ATMEL Microcontroller Software Support
 * ----------------------------------------------------------------------------
 * Copyright (c) 2008, Atmel Corporation
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer below.
 *
 * Atmel's name may not be used to endorse or promote products derived from
 * this software without specific prior written permission.
 *
 * DISCLAIMER: THIS SOFTWARE IS PROVIDED BY ATMEL "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT ARE
 * DISCLAIMED. IN NO EVENT SHALL ATMEL BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * ----------------------------------------------------------------------------
 */

//------------------------------------------------------------------------------
//         Headers
//------------------------------------------------------------------------------

#include "AUDIODriver.h"
#include "AUDIODriverDescriptors.h"
#include "AUDIOChannel.h"
#include "AUDIOGenericRequest.h"
#include "AUDIOFeatureUnitRequest.h"
#include "USBDDriver.h"
#include "doxaUSB.h"


extern void PrintByte(unsigned int c);

//------------------------------------------------------------------------------
//         Internal types
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
/// Audio speaker driver internal state.
//------------------------------------------------------------------------------
typedef struct {

    /// USB device driver instance.
    USBDDriver usbdDriver;
    /// List of AUDDSpeakerChannel instances for playback.
    AUDDSpeakerChannel channels[AUDIODriver_NUMCHANNELS+1];

} AUDDSpeakerDriver;

extern USBDDriver usbdDriver;

//------------------------------------------------------------------------------
//         Internal variables
//------------------------------------------------------------------------------

/// Global USB audio speaker driver instance.
static AUDDSpeakerDriver auddSpeakerDriver;
/// Array for storing the current setting of each interface.
static unsigned char auddSpeakerDriverInterfaces[2];
/// Intermediate storage variable for the mute status of a channel.
static unsigned char muted;

//------------------------------------------------------------------------------
//         Internal functions
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
/// Callback triggered after the new mute status of a channel has been read
/// by AUDDSpeakerDriver_SetFeatureCurrentValue. Changes the mute status
/// of the given channel accordingly.
/// \param channel Number of the channel whose mute status has changed.
//------------------------------------------------------------------------------
static void AUDDSpeakerDriver_MuteReceived(unsigned int channel)
{
	usart_write_line(USART1, "Mute received\n");
	//PrintByte(auddSpeakerDriver.channels[channel].muted);
	//PrintByte(channel);

    if (auddSpeakerDriver.channels[channel].muted) {
    	//auddSpeakerDriver.channels[channel].muted = 0;
    	//	set_mute(MUTE_OFF);
    	//usart_write_line(USART1, "Mute off\n");
        AUDDSpeakerChannel_Mute(&(auddSpeakerDriver.channels[channel]));
    }
    else {
    	//set_mute(MUTE_ON);
    	//usart_write_line(USART1, "Mute on\n");
    	//auddSpeakerDriver.channels[channel].muted = 1;
        AUDDSpeakerChannel_Unmute(&(auddSpeakerDriver.channels[channel]));
    }

    USBD_Write(0, 0, 0, 0, 0);
}


static void AUDIODriver_VolumeReceived(unsigned int channel){

	usart_write_line(USART1, "Volume changed\n");

	// Send ZLP
	USBD_Write(0, 0, 0, 0, 0);
}


//------------------------------------------------------------------------------
/// Sets the current value of a particular Feature control of a channel.
/// \param entity Entity number.
/// \param channel Channel number.
/// \param control Control selector value (see TODO).
/// \param length Length of the data containing the new value.
//------------------------------------------------------------------------------
static void AUDDSpeakerDriver_SetFeatureCurrentValue(unsigned char entity,
                                                     unsigned char channel,
                                                     unsigned char control,
                                                     unsigned short length)
{

	//	usart_write_line(USART1, "SetFeatureCurrentValue()\n");

//	PrintByte(entity);
//	PrintByte(channel);
//	PrintByte(control);
//	PrintByte(length);



	// Check the the requested control is supported
    // Control/channel combination not supported
    if ((control == AUDFeatureUnitRequest_MUTE)
        && (channel < (AUDIODriver_NUMCHANNELS+1))
        && (length == 1)) {

        unsigned int argument = channel; // Avoids compiler warning
        USBD_Read(0, // Endpoint #0
                  &muted,
                  sizeof(muted),
                  (TransferCallback) AUDDSpeakerDriver_MuteReceived,
                  (void *) argument);
    }

//    else if((control == AUDIOFeatureUnitRequest_VOLUME)
//		&& (channel < (AUDIODriver_NUMCHANNELS + 1))
//    	&& (length == 1)){
//
//    }

    // Control/channel combination not supported
    else {

        USBD_Stall(0);
    }
}

//------------------------------------------------------------------------------
/// Sends the current value of a particular channel Feature to the USB host.
/// \param entity Entity number.
/// \param channel Channel number.
/// \param control Control selector value (see TODO).
/// \param length Length of the data to return.
//------------------------------------------------------------------------------
static void AUDDSpeakerDriver_GetFeatureCurrentValue(unsigned char entity,
                                                     unsigned char channel,
                                                     unsigned char control,
                                                     unsigned char length)
{
//	usart_write_line(USART1, "GetFeatureCurrentValue()\n");
//	PrintByte(entity);
//	PrintByte(channel);
//	PrintByte(control);
//	PrintByte(length);


    // Check that the requested control is supported
    // Master channel mute control
    if ((control == AUDFeatureUnitRequest_MUTE)
        && (channel < (AUDIODriver_NUMCHANNELS+1))
        && (length == 1)) {

    	usart_write_line(USART1, "Sending mute status\n");
        muted = AUDDSpeakerChannel_IsMuted(&(auddSpeakerDriver.channels[channel]));
        USBD_Write(0, &muted, sizeof(muted), 0, 0);
    }

    else if ((control == AUDFeatureUnitRequest_VOLUME)
    	&& (channel < (AUDIODriver_NUMCHANNELS + 1))
    	&& (length == 2)){

    	usart_write_line(USART1, "Sending volume status\n");
//    	volume = AUDIOSpeakerChannel_GetVolume(&(auddSpeakerDriver.channels[channel]));
    	unsigned short int volume = Swap16(66);

    	USBD_Write(0, &volume, sizeof(volume), 0, 0);
    }

//  		  case AUDGenericRequest_GETMAX:
//            case AUDGenericRequest_GETMIN:
//            case AUDGenericRequest_GETRES:

    else {

        USBD_Stall(0);
    }
}

//------------------------------------------------------------------------------
//         Callback re-implementation
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
/// Triggered when the USB host emits a new SETUP request. The request is
/// forward to <AUDDSpeakerDriver_RequestHandler>.
/// \param request Pointer to a USBGenericRequest instance.
//------------------------------------------------------------------------------
void USBDCallbacks_RequestReceived(const USBGenericRequest *request)
{
    AUDDSpeakerDriver_RequestHandler(request);
}

//------------------------------------------------------------------------------
/// Invoked whenever the active setting of an interface is changed by the
/// host. Changes the status of the third LED accordingly.
/// \param interface Interface number.
/// \param setting Newly active setting.
//------------------------------------------------------------------------------
void USBDDriverCallbacks_InterfaceSettingChanged(unsigned char interface,
                                                 unsigned char setting)
{
    if ((interface == AUDIODriverDescriptors_STREAMING)
        && (setting == 0)) {

#ifdef DEBUG
    	usart_write_line(USART1, "Interface 0\n");
#endif
    }
    else {
#ifdef DEBUG
    	usart_write_line(USART1, "Interface 1\n");
#endif
    }
}

//------------------------------------------------------------------------------
//         Exported functions
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
/// Initializes an USB audio speaker device driver, as well as the underlying
/// USB controller.
//------------------------------------------------------------------------------
void AUDDSpeakerDriver_Initialize()
{
    // Initialize speaker channels
    AUDDSpeakerChannel_Initialize(&(auddSpeakerDriver.channels[0]),
                                  AUDDSpeakerDriver_MASTERCHANNEL,
                                  0);
    AUDDSpeakerChannel_Initialize(&(auddSpeakerDriver.channels[1]),
                                  AUDDSpeakerDriver_LEFTCHANNEL,
                                  0);
    AUDDSpeakerChannel_Initialize(&(auddSpeakerDriver.channels[2]),
                                  AUDDSpeakerDriver_RIGHTCHANNEL,
                                  0);

    // Initialize the USB driver
    USBDDriver_Initialize(&(auddSpeakerDriver.usbdDriver),
                          &audioDriverDescriptors,
                          auddSpeakerDriverInterfaces);
    USBD_Init();

}

//------------------------------------------------------------------------------
/// Handles audio-specific USB requests sent by the host, and forwards
/// standard ones to the USB device driver.
/// \param request Pointer to a USBGenericRequest instance.
//------------------------------------------------------------------------------
void AUDDSpeakerDriver_RequestHandler(const USBGenericRequest *request)
{
    unsigned char entity;
    unsigned char interface;

    // Check if this is a class request
    if (USBGenericRequest_GetType(request) == USBGenericRequest_CLASS) {

        // Check if the request is supported
        switch (USBGenericRequest_GetRequest(request)) {

            case AUDGenericRequest_SETCUR:

				usart_write_line(USART1, "Set cur\n");

                // Check the target interface and entity
                entity = AUDGenericRequest_GetEntity(request);
                interface = AUDGenericRequest_GetInterface(request);
                if ((entity == AUDIODriverDescriptors_FEATUREUNIT)
                    && (interface == AUDIODriverDescriptors_CONTROL)) {

                    AUDDSpeakerDriver_SetFeatureCurrentValue(
                        entity,
                        AUDFeatureUnitRequest_GetChannel(request),
                        AUDFeatureUnitRequest_GetControl(request),
                        USBGenericRequest_GetLength(request));
//                }
//                else if((entity == AUDIODriverDescriptors_INPUTTERMINAL)
//                		&& (interface == 5)){
//                	usart_write_line(USART1, "Entity: ");
//                	usart_putchar(USART1, entity + 48);
//                	usart_write_line(USART1, "\n");
//
//                	unsigned char input[3];
//                    USBD_Read(0, // Endpoint #0
//                              &input,
//                              sizeof(input),
//                              0,
//                              0);
                	//USBD_Write(0, 0, 0, 0, 0);
                    //USBD_Stall(0);
                }else{
//                	usart_write_line(USART1, "Entity: ");
//					usart_putchar(USART1, entity + 48);
//					usart_write_line(USART1, "Interface: ");
//					usart_putchar(USART1, interface + 48);
//					usart_write_line(USART1, "\n");
					USBD_Stall(0);
                }

                break;

            case AUDGenericRequest_GETCUR:

            	usart_write_line(USART1, "Get cur\n");

                // Check the target interface and entity
                entity = AUDGenericRequest_GetEntity(request);
                interface = AUDGenericRequest_GetInterface(request);
                if ((entity == AUDIODriverDescriptors_FEATUREUNIT)
                    && (interface == AUDIODriverDescriptors_CONTROL)) {

                    AUDDSpeakerDriver_GetFeatureCurrentValue(
                        entity,
                        AUDFeatureUnitRequest_GetChannel(request),
                        AUDFeatureUnitRequest_GetControl(request),
                        USBGenericRequest_GetLength(request));
                }


                else {

                    USBD_Stall(0);
                }
                break;



            default:

                USBD_Stall(0);
        }
    }
    // Check if this is a standard request
    else if (USBGenericRequest_GetType(request) == USBGenericRequest_STANDARD) {

        // Forward request to the standard handler
        USBDDriver_RequestHandler(&(auddSpeakerDriver.usbdDriver), request);

    }
    // Unsupported request type
    else {
    	usart_write_line(USART1, "Usup req: ");
		usart_putchar(USART1, USBGenericRequest_GetType(request) + 48);
		usart_putchar(USART1, '\n');

        USBD_Stall(0);
    }
}

//------------------------------------------------------------------------------
/// Reads incoming audio data sent by the USB host into the provided
/// buffer. When the transfer is complete, an optional callback function is
/// invoked.
/// \deprecated Introduces unnecessary overhead. Run USBD_Read
/// directly instead.
/// \param buffer Pointer to the data storage buffer.
/// \param length Size of the buffer in bytes.
/// \param callback Optional callback function.
/// \param argument Optional argument to the callback function.
/// \return USBD_STATUS_SUCCESS if the transfer is started successfully;
///         otherwise an error code.
//------------------------------------------------------------------------------
unsigned char AUDDSpeakerDriver_Read(void *buffer,
                                     unsigned int length,
                                     TransferCallback callback,
                                     void *argument)
{

//	usart_write_line(USART1, "Read audio data\n");

    return USBD_Read(AUDIODriverDescriptors_DATAOUT,	// Data out endpoint
                     buffer,							// Data out buffer
                     length,							// Length of data out buffer
                     callback,							// Callback function after read
                     argument);							// Argument to callback function
}

