
/*************************************
* Modified to support AP7 hardware
* By Espen Karlsen
**************************************/


/* ----------------------------------------------------------------------------
 *         ATMEL Microcontroller Software Support
 * ----------------------------------------------------------------------------
 * Copyright (c) 2008, Atmel Corporation
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer below.
 *
 * Atmel's name may not be used to endorse or promote products derived from
 * this software without specific prior written permission.
 *
 * DISCLAIMER: THIS SOFTWARE IS PROVIDED BY ATMEL "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT ARE
 * DISCLAIMED. IN NO EVENT SHALL ATMEL BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * ----------------------------------------------------------------------------
 */

//------------------------------------------------------------------------------
//         Headers
//------------------------------------------------------------------------------

#include "USBD.h"
#include "USBDCallbacks.h"
#include "USBDDriver.h"
#include <board.h>
#include <avr32/io.h>
#include "USBEndpointDescriptor.h"
#include "USBGenericRequest.h"
#include "USBFeatureRequest.h"
#include "usart.h"
#include "doxaUSB.h"
#include "compiler.h"

#include <stdio.h>


typedef struct avr32_usba_FIFO_t{
		 unsigned int		endpoint0[16384];	//0x00000
		 unsigned int		endpoint1[16384];
		 unsigned int		endpoint2[16384];
		 unsigned int		endpoint3[16384];
		 unsigned int		endpoint4[16384];
		 unsigned int		endpoint5[16384];
		 unsigned int		endpoint6[16384];
} avr32_usba_FIFO_t;



volatile avr32_usba_FIFO_t *usba_epfifo = &AVR32_USBA_FIFO;


//#ifdef BOARD_USB_UDPHS

//------------------------------------------------------------------------------
//         Definitions
//------------------------------------------------------------------------------

/// Maximum number of endpoints interrupts.
#define NUM_IT_MAX       \
    (usba->ipfeatures & AVR32_USBA_IPFEATURES_EPT_NBR_MAX_MASK)
/// Maximum number of endpoint DMA interrupts
#define NUM_IT_MAX_DMA   \
    ((usba->ipfeatures & AVR32_USBA_IPFEATURES_DMA_CHANNEL_NBR >> 4))
/// Bits that should be shifted to access DMA control bits.
#define SHIFT_DMA        24
/// Bits that should be shifted to access interrupt bits.
#define SHIFT_INTERUPT    8

/// Compile option, use DMA. Remove this define for not use DMA.
//#define DMA

/// Max size of the FMA FIFO
#define DMA_MAX_FIFO_SIZE 65536

//------------------------------------------------------------------------------
/// \page "Endpoint states"
/// This page lists the endpoint states.
/// !States
//  - UDP_ENDPOINT_DISABLED
//  - UDP_ENDPOINT_HALTED
//  - UDP_ENDPOINT_IDLE
//  - UDP_ENDPOINT_SENDING
//  - UDP_ENDPOINT_RECEIVING

/// Endpoint states: Endpoint is disabled
#define UDP_ENDPOINT_DISABLED       0
/// Endpoint states: Endpoint is halted (i.e. STALLs every request)
#define UDP_ENDPOINT_HALTED         1
/// Endpoint states: Endpoint is idle (i.e. ready for transmission)
#define UDP_ENDPOINT_IDLE           2
/// Endpoint states: Endpoint is sending data
#define UDP_ENDPOINT_SENDING        3
/// Endpoint states: Endpoint is receiving data
#define UDP_ENDPOINT_RECEIVING      4


//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
//      Structures
//------------------------------------------------------------------------------

/// Describes an ongoing transfer on a UDP endpoint.
typedef struct
{
    /// Pointer to a data buffer used for emission/reception.
    char             *pData;
    /// Number of bytes which have been written into the UDP internal FIFO
    /// buffers.
    volatile int     buffered;
    /// Number of bytes which have been sent/received.
    volatile int     transferred;
    /// Number of bytes which have not been buffered/transferred yet.
    volatile int     remaining;
    /// Optional callback to invoke when the transfer completes.
    volatile TransferCallback fCallback;
    /// Optional argument to the callback function.
    void             *pArgument;
} Transfer;

//------------------------------------------------------------------------------
/// Describes the state of an endpoint of the UDP controller.
//------------------------------------------------------------------------------
typedef struct
{
    /// Current endpoint state.
    volatile unsigned char  state;
    /// Current reception bank (0 or 1).
    unsigned char  bank;
    /// Maximum packet size for the endpoint.
    unsigned short size;
    /// Describes an ongoing transfer (if current state is either
    ///  <UDP_ENDPOINT_SENDING> or <UDP_ENDPOINT_RECEIVING>)
    Transfer       transfer;
} Endpoint;

//------------------------------------------------------------------------------
//         Internal variables
//------------------------------------------------------------------------------

/// Holds the internal state for each endpoint of the UDP.
static Endpoint      endpoints[BOARD_USB_NUMENDPOINTS];
/// Device current state.
static unsigned char deviceState;
/// Indicates the previous device state
static unsigned char previousDeviceState;
/// Special case for send a ZLP
static unsigned char sendZLP = 0;

/// 7.1.20 Test Mode Support
/// Test codes for the USB HS test mode.
static const char test_packet_buffer[] = {
    0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,                // JKJKJKJK * 9
    0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,                     // JJKKJJKK * 8
    0xEE,0xEE,0xEE,0xEE,0xEE,0xEE,0xEE,0xEE,                     // JJJJKKKK * 8
    0xFE,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF, // JJJJJJJKKKKKKK * 8
    0x7F,0xBF,0xDF,0xEF,0xF7,0xFB,0xFD,                          // JJJJJJJK * 8
    0xFC,0x7E,0xBF,0xDF,0xEF,0xF7,0xFB,0xFD,0x7E                 // {JKKKKKKK * 10}, JK
};

//------------------------------------------------------------------------------
//      Internal Functions
//------------------------------------------------------------------------------
void PrintByte(unsigned char c){

	char s[3];
	s[0] = c%10 + 48;
	s[1] = (c/10)%10 + 48;
	s[2] = (c/100)%10 + 48;

	usart_putchar(USART1, s[2]);
	usart_putchar(USART1, s[1]);
	usart_putchar(USART1, s[0]);
	usart_write_line(USART1, "\n");

}


//------------------------------------------------------------------------------
/// Disables the BIAS of the USB controller
//------------------------------------------------------------------------------
static inline void UDPHS_DisableBIAS( void ){}

//------------------------------------------------------------------------------
/// Enables the BIAS of the USB controller
//------------------------------------------------------------------------------
static inline void UDPHS_EnableBIAS( void ){}

//------------------------------------------------------------------------------
/// Enable UDPHS clock
//------------------------------------------------------------------------------
static inline void UDPHS_EnableUsbClock( void ){}

//------------------------------------------------------------------------------
/// Disable UDPHS clock
//------------------------------------------------------------------------------
static inline void UDPHS_DisableUsbClock( void ){}

//------------------------------------------------------------------------------
/// Handles a completed transfer on the given endpoint, invoking the
/// configured callback if any.
/// \param bEndpoint Number of the endpoint for which the transfer has completed.
/// \param bStatus   Status code returned by the transfer operation
//------------------------------------------------------------------------------
static void UDPHS_EndOfTransfer( unsigned char bEndpoint, char bStatus )
{
#ifdef DEBUG
	usart_write_line(USART1, "EOT ");
	PrintByte(bEndpoint);
#endif

    Endpoint *pEndpoint = &(endpoints[bEndpoint]);
    Transfer *pTransfer = &(pEndpoint->transfer);

    // Check that endpoint was sending or receiving data
    if( (pEndpoint->state == UDP_ENDPOINT_RECEIVING)
     || (pEndpoint->state == UDP_ENDPOINT_SENDING) ) {

    	//usart_write_line(USART1, "EndOfTransfer\n");

        // Endpoint returns in Idle state
        pEndpoint->state = UDP_ENDPOINT_IDLE;

        // Invoke callback is present
        if (pTransfer->fCallback != 0) {

        	//usart_write_line(USART1, "Invoke callback\n");

            ((TransferCallback) pTransfer->fCallback)
                (pTransfer->pArgument,
                 bStatus,
                 pTransfer->transferred,
                 pTransfer->remaining + pTransfer->buffered);
        }
        else {
				//usart_write_line(USART1, "No callback\n");
        }
    }
}

//------------------------------------------------------------------------------
/// Clears the correct RX flag in endpoint status register
/// \param bEndpoint Index of endpoint
//------------------------------------------------------------------------------
static void UDPHS_ClearRxFlag( unsigned char bEndpoint )
{
	usba->endpoint[bEndpoint].eptclrsta = AVR32_USBA_RX_BK_RDY_MASK;
}




//------------------------------------------------------------------------------
/// Transfers a data payload from the current tranfer buffer to the endpoint
/// FIFO
/// \param bEndpoint Number of the endpoint which is sending data.
//------------------------------------------------------------------------------
static void UDPHS_WritePayload( unsigned char bEndpoint )
{

#ifdef DEBUG
	usart_write_line(USART1, "Wrt Pld ");
	PrintByte(bEndpoint);
#endif

    Endpoint *pEndpoint = &(endpoints[bEndpoint]);
    Transfer *pTransfer = &(pEndpoint->transfer);
    char     *pFifo;
    signed int   size;
    unsigned int dCtr;

    //pFifo = (char*)((unsigned int *)AVR32_USBA_FIFO_ADDRESS + (AVR32_USBA_FIFO_SIZE * bEndpoint));
    switch (bEndpoint){
    case 5: pFifo = (char*)&usba_epfifo->endpoint5[0]; break;
    case 0: pFifo = (char*)&usba_epfifo->endpoint0[0]; break;
    case 1: pFifo = (char*)&usba_epfifo->endpoint1[0]; break;
    case 2: pFifo = (char*)&usba_epfifo->endpoint2[0]; break;
    case 3: pFifo = (char*)&usba_epfifo->endpoint3[0]; break;
    case 4: pFifo = (char*)&usba_epfifo->endpoint4[0]; break;
    case 6: pFifo = (char*)&usba_epfifo->endpoint6[0]; break;
    default: pFifo = (char*)&usba_epfifo->endpoint0[0];
    }


    // Get the number of bytes to send
    size = pEndpoint->size;
    if (size > pTransfer->remaining) {

        size = pTransfer->remaining;
    }

    // Update transfer descriptor information
    pTransfer->buffered += size;
    pTransfer->remaining -= size;

    // Write packet in the FIFO buffer
    dCtr = 0;
    while (size > 0) {
#ifdef DEBUG
    	usart_write_line(USART1, "Load FIFO: ");
    	PrintByte(*(pTransfer->pData));
#endif
        pFifo[dCtr] = *(pTransfer->pData);
        pTransfer->pData++;
        size--;
        dCtr++;
    }
}

//------------------------------------------------------------------------------
/// Transfers a data payload from an endpoint FIFO to the current transfer buffer
/// \param bEndpoint   Endpoint number.
/// \param wPacketSize Size of received data packet
//------------------------------------------------------------------------------
static void UDPHS_ReadPayload( unsigned char bEndpoint, int wPacketSize )
{
#ifdef DEBUG
	usart_write_line(USART1, "Rd pld ");
	PrintByte(bEndpoint);
#endif

    Endpoint *pEndpoint = &(endpoints[bEndpoint]);
    Transfer *pTransfer = &(pEndpoint->transfer);
    char     *pFifo;
    unsigned char dBytes=0;

    //pFifo = (char*)((unsigned int *)AVR32_USBA_FIFO_ADDRESS + (AVR32_USBA_FIFO_SIZE * bEndpoint));

    switch (bEndpoint){
    case 5: pFifo = (char*)&usba_epfifo->endpoint5[0]; break;
    case 0: pFifo = (char*)&usba_epfifo->endpoint0[0]; break;
    case 1: pFifo = (char*)&usba_epfifo->endpoint1[0]; break;
    case 2: pFifo = (char*)&usba_epfifo->endpoint2[0]; break;
    case 3: pFifo = (char*)&usba_epfifo->endpoint3[0]; break;
    case 4: pFifo = (char*)&usba_epfifo->endpoint4[0]; break;
    case 6: pFifo = (char*)&usba_epfifo->endpoint6[0]; break;
    default: pFifo = (char*)&usba_epfifo->endpoint0[0];
    }

    // Check that the requested size is not bigger than the remaining transfer
    if (wPacketSize > pTransfer->remaining) {

        pTransfer->buffered += wPacketSize - pTransfer->remaining;
        wPacketSize = pTransfer->remaining;
    }

    // Update transfer descriptor information
    pTransfer->remaining -= wPacketSize;
    pTransfer->transferred += wPacketSize;

    // Retrieve packet
    while (wPacketSize > 0) {

        *(pTransfer->pData) = pFifo[dBytes];
#ifdef DEBUG
        usart_putchar(USART1, *(pTransfer->pData)+48);
#endif
        pTransfer->pData++;
        wPacketSize--;
        dBytes++;
    }
}

//------------------------------------------------------------------------------
/// Received SETUP packet from endpoint 0 FIFO
/// \param pRequest Generic USB SETUP request sent over Control endpoints
//------------------------------------------------------------------------------
static void UDPHS_ReadRequest( USBGenericRequest *pRequest )
{

	unsigned int *pData = (unsigned int *)pRequest;
	unsigned int fifo;

	fifo = usba_epfifo->endpoint0[0];
	*pData = fifo;

	fifo = usba_epfifo->endpoint0[0];
	pData++;
	*pData = fifo;

  }


//------------------------------------------------------------------------------
/// Reset all endpoint transfer descriptors
//------------------------------------------------------------------------------
static void UDPHS_ResetEndpoints( void )
{
	usart_write_line(USART1, "Reset EP\n");

    Endpoint *pEndpoint;
    Transfer *pTransfer;
    unsigned char bEndpoint;

    // Reset the transfer descriptor of every endpoint
    for( bEndpoint = 0; bEndpoint < BOARD_USB_NUMENDPOINTS; bEndpoint++ ) {

        pEndpoint = &(endpoints[bEndpoint]);
        pTransfer = &(pEndpoint->transfer);

        // Reset endpoint transfer descriptor
        pTransfer->pData = 0;
        pTransfer->transferred = -1;
        pTransfer->buffered = -1;
        pTransfer->remaining = -1;
        pTransfer->fCallback = 0;
        pTransfer->pArgument = 0;

        // Reset endpoint state
        pEndpoint->bank = 0;
        pEndpoint->state = UDP_ENDPOINT_DISABLED;
    }
}

//------------------------------------------------------------------------------
/// Disable all endpoints (except control endpoint 0), aborting current
/// transfers if necessary
//------------------------------------------------------------------------------
static void UDPHS_DisableEndpoints( void )
{
	//usart_write_line(USART1, "Disable EP\n");
    unsigned char bEndpoint;

    // Disable each endpoint, terminating any pending transfer
    // Control endpoint 0 is not disabled
    for( bEndpoint = 1; bEndpoint < BOARD_USB_NUMENDPOINTS; bEndpoint++ ) {

        UDPHS_EndOfTransfer( bEndpoint, USBD_STATUS_ABORTED );
        endpoints[bEndpoint].state = UDP_ENDPOINT_DISABLED;
    }
}

//------------------------------------------------------------------------------
/// Endpoint interrupt handler.
/// Handle IN/OUT transfers, received SETUP packets and STALLing
/// \param bEndpoint Index of endpoint
//------------------------------------------------------------------------------
static void UDPHS_EndpointHandler( unsigned char bEndpoint )
{

#ifdef DEBUG
	usart_write_line(USART1, "EP handler: ");
	PrintByte(bEndpoint);
#endif

	Endpoint *pEndpoint = &(endpoints[bEndpoint]);
    Transfer *pTransfer = &(pEndpoint->transfer);
    unsigned int   status = usba->endpoint[bEndpoint].eptsta;
    unsigned short wPacketSize;
    USBGenericRequest request;

    // Handle interrupts
    // IN packet sent
    if( (AVR32_USBA_TX_PK_RDY_MASK == (usba->endpoint[bEndpoint].eptctl & AVR32_USBA_TX_PK_RDY_MASK))
    	&& (0 == (status & AVR32_USBA_TX_PK_RDY_MASK)) ){

#ifdef DEBUG
    	usart_write_line(USART1, "In packet \n");
#endif

        // Check that endpoint was in Sending state
        if( pEndpoint->state == UDP_ENDPOINT_SENDING ) {

            if (pTransfer->buffered > 0) {
                pTransfer->transferred += pTransfer->buffered;
                pTransfer->buffered = 0;
            }

            if(  ((pTransfer->buffered)==0)
               &&((pTransfer->transferred)==0)
               &&((pTransfer->remaining)==0)
               &&(sendZLP == 0)) {
                sendZLP = 1;
            }

            // End of transfer ?
            if( (pTransfer->remaining > 0) || (sendZLP == 1)) {

                // Send next packet
                UDPHS_WritePayload(bEndpoint);
                usba->endpoint[bEndpoint].eptsetsta = AVR32_USBA_TX_PK_RDY_MASK;
                sendZLP = 2;
            }
            else {

            	// Disable interrupt if this is not a control endpoint
                if ( AVR32_USBA_EPT_TYPE_CONTROL != ( AVR32_USBA_EPT_TYPE_MASK & usba->endpoint[bEndpoint].eptcfg) ){

                	usba->ien &= ~(1 << AVR32_USBA_IEN_EPT_INT_0_OFFSET << bEndpoint);
                }

                usba->endpoint[bEndpoint].eptctldis = AVR32_USBA_TX_PK_RDY_MASK;

                UDPHS_EndOfTransfer(bEndpoint, USBD_STATUS_SUCCESS);
                sendZLP = 0;
            }
        }
        else {
        	usart_write_line(USART1, "Error1\n");
        }
    }

    // OUT packet received at given endpoint
    if ( AVR32_USBA_RX_BK_RDY_MASK == (status & AVR32_USBA_RX_BK_RDY_MASK) ){

#ifdef DEBUG
    	usart_write_line(USART1, "Out packet\n");
#endif

        // Check that the endpoint is in Receiving state
        if (pEndpoint->state != UDP_ENDPOINT_RECEIVING) {

            // Check if an ACK has been received on a Control endpoint
        	if( (0 == (usba->endpoint[bEndpoint].eptcfg & AVR32_USBA_EPT_TYPE_MASK))
        			&& (0 == (status & AVR32_USBA_BYTE_COUNT_MASK)) ){

                // Control endpoint, 0 bytes received
                // Acknowledge the data and finish the current transfer
        		usart_write_line(USART1, "Ctrl ep - Ack\n");
                UDPHS_ClearRxFlag(bEndpoint);
                UDPHS_EndOfTransfer(bEndpoint, USBD_STATUS_SUCCESS);
            }

            // Check if the data has been STALLed
        	else if (AVR32_USBA_FRCESTALL_MASK == (status & AVR32_USBA_FRCESTALL_MASK)){

                // Discard STALLed data
        		usart_write_line(USART1, "Discard\n");
                UDPHS_ClearRxFlag(bEndpoint);
            }
            // NAK the data
            else {
            	usart_write_line(USART1, "Nak\n");
                usba->ien &= ~(1 << AVR32_USBA_IEN_EPT_INT_0_OFFSET << bEndpoint);
            }
        }
        else {

#ifdef DEBUG
        	usart_putchar(USART1, bEndpoint + 48);
        	usart_write_line(USART1, "EP r\n");
#endif
            // Endpoint is in Read state
            // Retrieve data and store it into the current transfer buffer
            wPacketSize = (unsigned short)((status & AVR32_USBA_BYTE_COUNT_MASK ) >>AVR32_USBA_BYTE_COUNT_OFFSET);

//           PrintByte(wPacketSize);

            UDPHS_ReadPayload(bEndpoint, wPacketSize);
            UDPHS_ClearRxFlag(bEndpoint);

            // Check if the transfer is finished
            if ((pTransfer->remaining == 0) || (wPacketSize < pEndpoint->size)) {

            	usba->endpoint[bEndpoint].eptctldis = AVR32_USBA_RX_BK_RDY_MASK;

                // Disable interrupt if this is not a control endpoint
                if (AVR32_USBA_EPT_TYPE_CONTROL != (AVR32_USBA_EPT_TYPE_MASK & usba->endpoint[bEndpoint].eptcfg)){

                	usba->ien &= ~(1 << AVR32_USBA_IEN_EPT_INT_0_OFFSET << bEndpoint);
                }
                UDPHS_EndOfTransfer(bEndpoint, USBD_STATUS_SUCCESS);

//				usart_putchar(USART1, *(pEndpoint->transfer.pData + 2) + 48);
#ifdef DEBUG
                usart_write_line(USART1, ".TF fin\n");
#endif
            }
        }
    }


    // STALL sent
    if ( AVR32_USBA_STALL_SNT_MASK == (status & AVR32_USBA_STALL_SNT_MASK)){

        // Acknowledge the stall flag
        usba->endpoint[bEndpoint].eptclrsta = AVR32_USBA_STALL_SNT_MASK;

        // If the endpoint is not halted, clear the STALL condition
        if (pEndpoint->state != UDP_ENDPOINT_HALTED) {

            usba->endpoint[bEndpoint].eptclrsta = AVR32_USBA_FRCESTALL_MASK;
        }
    }

    // SETUP packet received
    if (AVR32_USBA_RX_SETUP_MASK == (status & AVR32_USBA_RX_SETUP_MASK)){

#ifdef DEBUG
    	usart_write_line(USART1, "Setup\n");
#endif

        // If a transfer was pending, complete it
        // Handles the case where during the status phase of a control write
        // transfer, the host receives the device ZLP and ack it, but the ack
        // is not received by the device
        if ((pEndpoint->state == UDP_ENDPOINT_RECEIVING)
            || (pEndpoint->state == UDP_ENDPOINT_SENDING)) {

            UDPHS_EndOfTransfer(bEndpoint, USBD_STATUS_SUCCESS);
        }
        // Copy the setup packet
        UDPHS_ReadRequest(&request);

        // Acknowledge setup packet
        usba->endpoint[bEndpoint].eptclrsta = AVR32_USBA_RX_SETUP_MASK;

        // Forward the request to the upper layer
        USBDCallbacks_RequestReceived(&request);
    }

}


//------------------------------------------------------------------------------
/// Endpoint interrupt handler.
/// Handle IN/OUT transfers, received SETUP packets and STALLing
/// \param bEndpoint Index of endpoint
//------------------------------------------------------------------------------
static void UDPHS_DATA_OUT(unsigned char bEndpoint)
{

	unsigned int   status = usba->endpoint[bEndpoint].eptsta;
    unsigned short wPacketSize;
    char     *pFifo = (char*)&usba_epfifo->endpoint5[0];
    unsigned char dBytes=0;

    // OUT packet received at given endpoint
//    if ( AVR32_USBA_RX_BK_RDY_MASK == (status & AVR32_USBA_RX_BK_RDY_MASK) ){

//			usart_write_line(USART1, "RX_BK_RDY\n");
            // Endpoint is in Read state
            wPacketSize = (unsigned short)((status & AVR32_USBA_BYTE_COUNT_MASK ) >>AVR32_USBA_BYTE_COUNT_OFFSET);

			// Retrieve packet
			while (wPacketSize > 0){
				aud_buf[usb_buf_indx][dBytes] = pFifo[dBytes];
				dBytes++;
				wPacketSize--;
			}

			aud_buf_sizes[usb_buf_indx] = dBytes;
			usb_buf_indx = (usb_buf_indx + 1) % AUDIO_BUFFER_COUNT;
			aud_buf_rdy = (aud_buf_rdy + 1) % AUDIO_BUFFER_COUNT;

            UDPHS_ClearRxFlag(bEndpoint);

            usba->endpoint[bEndpoint].eptclrsta = AVR32_USBA_RX_SETUP_MASK;
}

//------------------------------------------------------------------------------
//      Interrupt service routine
//------------------------------------------------------------------------------
#ifdef DMA
//----------------------------------------------------------------------------
/// Endpoint DMA interrupt handler.
/// This function (ISR) handles dma interrupts
/// \param bEndpoint Index of endpoint
//----------------------------------------------------------------------------
static void UDPHS_DmaHandler( unsigned char bEndpoint )
{
	usart_write_line(USART1, "m\n");
    Endpoint     *pEndpoint = &(endpoints[bEndpoint]);
    Transfer     *pTransfer = &(pEndpoint->transfer);
    int           justTransferred;
    unsigned int  status;
    unsigned char result = USBD_STATUS_SUCCESS;

    status = usba->dma[bEndpoint].dmastatus;

    // Disable DMA interrupt to avoid receiving 2 interrupts (B_EN and TR_EN)
    usba->dma[bEndpoint].dmacontrol &= ~(AVR32_USBA_END_TR_EN_MASK | AVR32_USBA_END_B_EN_MASK);


    usba->ien &= ~(1 << SHIFT_DMA << bEndpoint);

    if( AVR32_USBA_END_BF_ST_MASK == (status & AVR32_USBA_END_BF_ST_MASK)){


        // BUFF_COUNT holds the number of untransmitted bytes.
        // BUFF_COUNT is equal to zero in case of good transfer
        justTransferred = pTransfer->buffered
								   - ((status & AVR32_USBA_BUFF_COUNT_MASK) >> AVR32_USBA_BUFF_COUNT_OFFSET);

        pTransfer->transferred += justTransferred;

        pTransfer->buffered = ((status & AVR32_USBA_BUFF_COUNT_MASK) >> AVR32_USBA_BUFF_COUNT_OFFSET);

        pTransfer->remaining -= justTransferred;


        if( (pTransfer->remaining + pTransfer->buffered) > 0 ) {

            // Prepare an other transfer
            if( pTransfer->remaining > DMA_MAX_FIFO_SIZE ) {

                pTransfer->buffered = DMA_MAX_FIFO_SIZE;
            }
            else {
                pTransfer->buffered = pTransfer->remaining;
            }

            usba->dma[bEndpoint].dmaaddress =
                (unsigned int)((pTransfer->pData) + (pTransfer->transferred));

            // Clear unwanted interrupts
            usba->dma[bEndpoint].dmastatus;

            // Enable DMA endpoint interrupt
            usba->ien |= (1 << SHIFT_DMA << bEndpoint);

            // DMA config for receive the good size of buffer, or an error buffer

            usba->dma[bEndpoint].dmacontrol = 0;

            usba->dma[bEndpoint].dmacontrol =
									( ((pTransfer->buffered << 16) & AVR32_USBA_BUFF_COUNT_MASK)
										| AVR32_USBA_END_TR_EN_MASK
										| AVR32_USBA_END_TR_IT_MASK
										| AVR32_USBA_END_B_EN_MASK
										| AVR32_USBA_END_BUFFIT_MASK
										| AVR32_USBA_CHANN_ENB_MASK );
        }
    }
    else if ( AVR32_USBA_END_TR_ST_MASK == (status & AVR32_USBA_END_TR_ST_MASK)){

        usart_write_line(USART1, "Eot\n");

        pTransfer->transferred = pTransfer->buffered
								 - ((status & AVR32_USBA_BUFF_COUNT_MASK) >> AVR32_USBA_BUFF_COUNT_OFFSET);
        pTransfer->remaining = 0;
    }
    else {

        usart_write_line(USART1, "DMA error\n");
        result = USBD_STATUS_ABORTED;
    }

    // Invoke callback
    if( pTransfer->remaining == 0 ) {

        UDPHS_EndOfTransfer(bEndpoint, result);
    }
}
#endif

//------------------------------------------------------------------------------
//      Exported functions
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
/// USB interrupt handler
/// Manages device resume, suspend, end of bus reset.
/// Forwards endpoint interrupts to the appropriate handler.
//------------------------------------------------------------------------------
__attribute__((__interrupt__)) void USBD_InterruptHandler(void)
{

#ifdef DEBUG
	usart_write_line(USART1, "USBD_InterruptHandler\n");
#endif

    unsigned int  status;
    unsigned char numIT;

    if (deviceState >= USBD_STATE_POWERED) {
    	//usart_write_line(USART1, "powered\n");
    }

    // Get interrupts status
    status = usba->intsta & usba->ien;

    // Handle all USBA interrupts
   while (status != 0) {


        // Start Of Frame (SOF)
//        if ((status & AVR32_USBA_IEN_INT_SOF_MASK) != 0) {
//
//        	usart_write_line(USART1, "irq - INT_SOF\n");
//
//            // Invoke the SOF callback
//            //USB_StartOfFrameCallback(pUsb);
//
//            // Acknowledge interrupt
//            usba->clrint = AVR32_USBA_IEN_INT_SOF_MASK;
//
//            // Clear SOF interrupt status
//            status &= ~AVR32_USBA_IEN_INT_SOF_MASK;
//        }


//        else if ((status & AVR32_USBA_IEN_MICRO_SOF_MASK) != 0){
//        	usart_write_line(USART1, "irq - INT_MICRO_SOF\n");
//        	usba->clrint |= AVR32_USBA_IEN_MICRO_SOF_MASK;
//        	status &= ~AVR32_USBA_IEN_MICRO_SOF_MASK;
//        }

        // Suspend
        // This interrupt is always treated last (hence the '==')
        if (status == AVR32_USBA_DET_SUSPD_MASK){

        	usart_write_line(USART1, "irq - DET_SUSPD\n");

            // The device enters the Suspended state
            // MCK + UDPCK must be off
            // Pull-Up must be connected
            // Transceiver must be disabled

            // Enable wakeup
            usba->ien |= AVR32_USBA_WAKE_UP_MASK | AVR32_USBA_ENDOFRSM_MASK; //$ | AVR32_USBA_MICRO_SOF_MASK;
            usba->ien &= ~AVR32_USBA_DET_SUSPD_MASK;

            // Acknowledge interrupt
            usba->clrint |= AVR32_USBA_DET_SUSPD_MASK | AVR32_USBA_WAKE_UP_MASK;
            previousDeviceState = deviceState;
            deviceState = USBD_STATE_SUSPENDED;

            // Invoke the Suspend callback
            USBDCallbacks_Suspended();
        }

        // Resume
        else if( ((status & AVR32_USBA_WAKE_UP_MASK) != 0)
        		|| ((status & AVR32_USBA_ENDOFRSM_MASK) != 0)){

        	usart_write_line(USART1, "Resumed\n");

                // Invoke the Resume callback
                USBDCallbacks_Resumed();

                // The device enters Configured state
                // MCK + UDPCK must be on
                // Pull-Up must be connected
                // Transceiver must be enabled


                deviceState = previousDeviceState;

                // Clear interrupt flags
                usba->clrint 	= AVR32_USBA_WAKE_UP_MASK | AVR32_USBA_ENDOFRSM_MASK | AVR32_USBA_DET_SUSPD_MASK;

                // Enable interrupts
                usba->ien 	   |= AVR32_USBA_ENDOFRSM_MASK | AVR32_USBA_DET_SUSPD_MASK;

                // Clear interrupt flags
                usba->clrint 	= AVR32_USBA_WAKE_UP_MASK | AVR32_USBA_ENDOFRSM_MASK;

                // Enable wake up interrupt
                usba->ien 	   &= ~AVR32_USBA_WAKE_UP_MASK;

        }


        // End of bus reset
        else if (( status & AVR32_USBA_ENDRESET_MASK) == AVR32_USBA_ENDRESET_MASK){

			usart_write_line(USART1, "irq - Endreset\n");

            // The device enters the Default state
            deviceState = USBD_STATE_DEFAULT;

            usart_write_line(USART1, "st - default\n");
            //      MCK + UDPCK are already enabled
            //      Pull-Up is already connected
            //      Transceiver must be enabled
            //      Endpoint 0 must be enabled

            // Reset all endpoints
            UDPHS_ResetEndpoints();

            // Disable all endpoints
            UDPHS_DisableEndpoints();

            // Configure control endpoint
            USBD_ConfigureEndpoint(0);

            // Flush and enable the Suspend interrupt
            usba->clrint = AVR32_USBA_WAKE_UP_MASK | AVR32_USBA_DET_SUSPD_MASK;


            //// Enable the Start Of Frame (SOF) interrupt if needed
            //if (pCallbacks->startOfFrame != 0)
            //{
            	//usart_write_line(USART1, "SOF irq enabled\n");
				//usba->ien |= AVR32_USBA_IEN_INT_SOF_MASK; // Lagt til denne, EK
            //}

            // Invoke the Reset callback
            USBDCallbacks_Reset();

            // Acknowledge end of bus reset interrupt
            usba->clrint = AVR32_USBA_ENDRESET_MASK;

            usba->ien |= AVR32_USBA_DET_SUSPD_MASK;

        }

        // Handle upstream resume interrupt
        else if (status & AVR32_USBA_UPSTR_RES_MASK){

        	usart_write_line(USART1, "irq - UPSTR_RES\n");

            // - Acknowledge the IT
        	usba->clrint = AVR32_USBA_UPSTR_RES_MASK;
        }

        // Endpoint interrupts
        else {
        	//usart_write_line(USART1, "irq - Endpoint\n");
#ifndef DMA
            // Handle endpoint interrupts
            for (numIT = 0; numIT < NUM_IT_MAX; numIT++) {

                if ((status & (1 << AVR32_USBA_EPT_INT_0_OFFSET << numIT)) != 0) {

                    // OUT packet received at given endpoint
                    UDPHS_EndpointHandler(numIT);

                }
            }
#else
            // Handle endpoint control interrupt
            if ((status & (1 << AVR32_USBA_EPT_INT_0_OFFSET << 0)) != 0) {

//            	usart_write_line(USART1, "irq - EP control\n");
                UDPHS_EndpointHandler( 0 );
            }
            else {
            	usart_write_line(USART1, "d\n");
//            	usart_write_line(USART1, "irq - EP not control\n");
                numIT = 1;
                while((status & (0x7E << SHIFT_DMA)) != 0) {

                    // Check if endpoint has a pending interrupt
                    if ((status & (1 << SHIFT_DMA << numIT)) != 0) {

                        UDPHS_DmaHandler(numIT);
                        status &= ~(1 << SHIFT_DMA << numIT);

//                        if (status != 0) {
//
//                        }
                    }
                    numIT++;
                }
            }
#endif
        }

        // Retrieve new interrupt status
        status = usba->intsta & usba->ien;

    }

    if (deviceState >= USBD_STATE_POWERED) {
    	//usart_write_line(USART1, "st - Powered\n");
    }
}

//------------------------------------------------------------------------------
/// Configure an endpoint with the provided endpoint descriptor
/// \param pDdescriptor Pointer to the endpoint descriptor
//------------------------------------------------------------------------------
void USBD_ConfigureEndpoint(const USBEndpointDescriptor *pDescriptor)
{

    Endpoint *pEndpoint;
    unsigned char bEndpoint;
    unsigned char bType;
    unsigned char bEndpointDir;
    unsigned char bSizeEpt = 0;

    // NULL descriptor -> Control endpoint 0
    if (pDescriptor == 0) {

        bEndpoint = 0;
        pEndpoint = &(endpoints[bEndpoint]);
        bType = USBEndpointDescriptor_CONTROL;
        bEndpointDir = 0;
        pEndpoint->size = BOARD_USB_ENDPOINTS_MAXPACKETSIZE(0);
        pEndpoint->bank = BOARD_USB_ENDPOINTS_BANKS(0);
    }
    else  {

        // The endpoint number
        bEndpoint = USBEndpointDescriptor_GetNumber(pDescriptor);
        pEndpoint = &(endpoints[bEndpoint]);
        // Transfer type: Control, Isochronous, Bulk, Interrupt
        bType = USBEndpointDescriptor_GetType(pDescriptor);
        // Direction, ignored for control endpoints
        bEndpointDir = USBEndpointDescriptor_GetDirection(pDescriptor);
        pEndpoint->size = Swap16(USBEndpointDescriptor_GetMaxPacketSize(pDescriptor));
        pEndpoint->bank = BOARD_USB_ENDPOINTS_BANKS(bEndpoint);

    }

    // Abort the current transfer is the endpoint was configured and in
    // Write or Read state
    if( (pEndpoint->state == UDP_ENDPOINT_RECEIVING)
     || (pEndpoint->state == UDP_ENDPOINT_SENDING) ) {

        UDPHS_EndOfTransfer(bEndpoint, USBD_STATUS_RESET);
    }

    pEndpoint->state = UDP_ENDPOINT_IDLE;
    usart_write_line(USART1, "st - Idle\n");

    // Disable endpoint
    usba->endpoint[bEndpoint].eptctldis = AVR32_USBA_SHRT_PCKT_MASK
										| AVR32_USBA_BUSY_BANK_MASK
										| AVR32_USBA_NAK_OUT_MASK
										| AVR32_USBA_NAK_IN_MASK
										| AVR32_USBA_STALL_SNT_MASK
										| AVR32_USBA_RX_SETUP_MASK
										| AVR32_USBA_TX_PK_RDY_MASK
										| AVR32_USBA_TX_COMPLT_MASK
										| AVR32_USBA_RX_BK_RDY_MASK
										| AVR32_USBA_ERR_OVFLW_MASK
										| AVR32_USBA_MDATA_RX_MASK
										| AVR32_USBA_DATAX_RX_MASK
										| AVR32_USBA_NYET_DIS_MASK
										| AVR32_USBA_INTDIS_DMA_MASK
										| AVR32_USBA_AUTO_VALID_MASK
										| AVR32_USBA_EPT_DISABL_MASK;


    // Reset Endpoint Fifos
    usba->endpoint[bEndpoint].eptclrsta = AVR32_USBA_TOGGLESQ_MASK | AVR32_USBA_FRCESTALL_MASK;

    usba->eptrst = 1<<bEndpoint;


    // Configure endpoint
    if( pEndpoint->size <= 8 )  {
        bSizeEpt = 0;
    }
    else if ( pEndpoint->size <= 16 ) {
        bSizeEpt = 1;
    }
    else if ( pEndpoint->size <= 32 ) {
        bSizeEpt = 2;
    }
    else if ( pEndpoint->size <= 64 ) {
        bSizeEpt = 3;
        usart_write_line(USART1, "64\n");
    }
    else if ( pEndpoint->size <= 128 ) {
        bSizeEpt = 4;
    }
    else if ( pEndpoint->size <= 256 ) {
        bSizeEpt = 5;
    }
    else if ( pEndpoint->size <= 512 )  {
        bSizeEpt = 6;
    }
    else if ( pEndpoint->size <= 1024 ) {
        bSizeEpt = 7;

#ifdef DEBUG
        usart_write_line(USART1, "1024\n");
        PrintByte(pEndpoint->bank);
        PrintByte(bType);
#endif

    } else {
      bSizeEpt = 0; // control endpoint
    }

    // Configure endpoint
    if (bType == USBEndpointDescriptor_CONTROL) {

        // Enable endpoint IT for control endpoint
        usba->ien |= (1 << AVR32_USBA_EPT_INT_0_OFFSET << bEndpoint);

    }

	usba->endpoint[bEndpoint].eptcfg = bSizeEpt
									 | (bEndpointDir << AVR32_USBA_EPT_DIR_OFFSET)
									 | (bType << AVR32_USBA_EPT_TYPE_OFFSET)
									 | ((pEndpoint->bank)<< AVR32_USBA_BK_NUMBER_OFFSET);

	while ( (signed int)AVR32_USBA_EPT_MAPD_MASK != (signed int)((usba->endpoint[bEndpoint].eptcfg) & AVR32_USBA_EPT_MAPD_MASK) ){
#ifdef DEBUG
		usart_write_line(USART1, "EP not mapped\n");
        for(;;); // Why the looping? Infinity makes me dizzy ...
#endif
    }

	#ifdef DEBUG
	usart_write_line(USART1, "EP ");
	usart_putchar(USART1, (bEndpoint+48));
	usart_write_line(USART1, " mapped\n");
#endif

    if (bType == USBEndpointDescriptor_CONTROL) {

    	usba->endpoint[bEndpoint].eptctlenb = AVR32_USBA_RX_BK_RDY_MASK
											| AVR32_USBA_RX_SETUP_MASK
											| AVR32_USBA_EPT_ENABL_MASK;

    }
    else {
#ifndef DMA
    	PrintByte(bEndpoint);
    	usart_write_line(USART1, " bType = NOT control\n");
    	usba->endpoint[bEndpoint].eptctlenb = AVR32_USBA_EPT_ENABL_MASK;
#else
    	usart_write_line(USART1, "bType = NOT control\n");
        usba->endpoint[bEndpoint].eptctlenb = AVR32_USBA_AUTO_VALID_MASK | AVR32_USBA_EPT_ENABL_MASK;
#endif
    }
}

//------------------------------------------------------------------------------
/// Sends data through an USB endpoint (IN)
/// Sets up the transfer descriptor, write one or two data payloads
/// (depending on the number of FIFO banks for the endpoint) and then
/// starts the actual transfer. The operation is complete when all
/// the data has been sent.
/// \param bEndpoint Index of endpoint
/// \param *pData  Data to be written
/// \param dLength Data length to be send
/// \param fCallback  Callback to be call after the success command
/// \param *pArgument Callback argument
/// \return USBD_STATUS_LOCKED or USBD_STATUS_SUCCESS
//------------------------------------------------------------------------------
char USBD_Write( unsigned char    bEndpoint,
                 const void       *pData,
                 unsigned int     dLength,
                 TransferCallback fCallback,
                 void             *pArgument )
{
    Endpoint *pEndpoint = &(endpoints[bEndpoint]);
    Transfer *pTransfer = &(pEndpoint->transfer);

    // Return if the endpoint is not in IDLE state
    if (pEndpoint->state != UDP_ENDPOINT_IDLE)  {

        return USBD_STATUS_LOCKED;
    }

    //TRACE_DEBUG_WP("Write%d(%d) ", bEndpoint, dLength);

    // Setup the transfer descriptor
    pTransfer->pData = (void *) pData;
    pTransfer->remaining = dLength;
    pTransfer->buffered = 0;
    pTransfer->transferred = 0;
    pTransfer->fCallback = fCallback;
    pTransfer->pArgument = pArgument;

    // Send one packet
    pEndpoint->state = UDP_ENDPOINT_SENDING;

#ifdef DMA
    // Test if endpoint type control
    if (AVR32_USBA_EPT_TYPE_CONTROL == (AVR32_USBA_EPT_TYPE_MASK & (usba->endpoint[bEndpoint].eptcfg))){
#endif
        // Enable endpoint IT
    	usba->ien |= (1 << AVR32_USBA_EPT_INT_0_OFFSET << bEndpoint);

    	usba->endpoint[bEndpoint].eptctlenb = AVR32_USBA_TX_PK_RDY_MASK;

#ifdef DMA
    }
    else {

        if( pTransfer->remaining == 0 ) {

        	// DMA not handle ZLP
            usba->endpoint[bEndpoint].eptsetsta = AVR32_USBA_TX_PK_RDY_MASK;

            // Enable endpoint IT
            usba->ien |= (1 << AVR32_USBA_EPT_INT_0_OFFSET << bEndpoint);

            usba->endpoint[bEndpoint].eptctlenb = AVR32_USBA_TX_PK_RDY_MASK;

        }
        else {
            // Others endpoints (not control)
            if( pTransfer->remaining > DMA_MAX_FIFO_SIZE ) {

                // Transfer the max
                pTransfer->buffered = DMA_MAX_FIFO_SIZE;
            }
            else {
                // Transfer the good size
                pTransfer->buffered = pTransfer->remaining;
            }

            usba->dma[bEndpoint].dmaaddress = (unsigned int)(pTransfer->pData);

            // Clear unwanted interrupts
            usba->dma[bEndpoint].dmastatus;

            // Enable DMA endpoint interrupt
            usba->ien |= (1 << AVR32_USBA_EPT_INT_0_OFFSET << bEndpoint);

            // DMA config
			usba->dma[bEndpoint].dmacontrol = 0;


			usba->dma[bEndpoint].dmacontrol =
											( ((pTransfer->buffered << 16) & AVR32_USBA_BUFF_COUNT_MASK)
												| AVR32_USBA_END_B_EN_MASK
												| AVR32_USBA_END_BUFFIT_MASK
												| AVR32_USBA_CHANN_ENB_MASK);
        }
    }
#endif

    return USBD_STATUS_SUCCESS;
}

//------------------------------------------------------------------------------
/// Reads incoming data on an USB endpoint (OUT)
/// \param bEndpoint Index of endpoint
/// \param *pData  Data to be read
/// \param dLength Data length to be receive
/// \param fCallback  Callback to be call after the success command
/// \param *pArgument Callback argument
/// \return USBD_STATUS_LOCKED or USBD_STATUS_SUCCESS
//------------------------------------------------------------------------------
char USBD_Read( unsigned char    bEndpoint,
                void             *pData,
                unsigned int     dLength,
                TransferCallback fCallback,
                void             *pArgument )
{
    Endpoint *pEndpoint = &(endpoints[bEndpoint]);
    Transfer *pTransfer = &(pEndpoint->transfer);

    // Return if the endpoint is not in IDLE state
    if (pEndpoint->state != UDP_ENDPOINT_IDLE) {

//    	usart_write_line(USART1, "Status locked\n");
        return USBD_STATUS_LOCKED;
    }


    // Endpoint enters Receiving state
    pEndpoint->state = UDP_ENDPOINT_RECEIVING;
//    usart_write_line(USART1, "Status receiving\n");


    // Set the transfer descriptor
    pTransfer->pData = pData;
    pTransfer->remaining = dLength;
    pTransfer->buffered = 0;
    pTransfer->transferred = 0;
    pTransfer->fCallback = fCallback;
    pTransfer->pArgument = pArgument;

#ifdef DMA
    // Test if endpoint type control
    if (AVR32_USBA_EPT_TYPE_CONTROL == (AVR32_USBA_EPT_TYPE_MASK & (usba->endpoint[bEndpoint].eptcfg))){

#endif
        // Control endpoint
        // Enable endpoint IT
        usba->ien |= (1 << AVR32_USBA_EPT_INT_0_OFFSET << bEndpoint);
        usba->endpoint[bEndpoint].eptctlenb |= AVR32_USBA_RX_BK_RDY_MASK;

#ifdef DMA
    }
    else {

        // Others endpoints (not control)
        if( pTransfer->remaining > DMA_MAX_FIFO_SIZE ) {

            // Transfer the max
            pTransfer->buffered = DMA_MAX_FIFO_SIZE;
        }
        else {
            // Transfer the good size
            pTransfer->buffered = pTransfer->remaining;
        }

        usba->dma[bEndpoint].dmaaddress = (unsigned int)(pTransfer->pData);

        // Clear unwanted interrupts
        usba->dma[bEndpoint].dmastatus;

        // Enable DMA endpoint interrupt
        usba->ien |= (1 << AVR32_USBA_DMA_INT_1_OFFSET << bEndpoint);

        // DMA config
        usba->dma[bEndpoint].dmacontrol = 0;

        usba->dma[bEndpoint].dmacontrol =
								( ((pTransfer->buffered << 16) & AVR32_USBA_BUFF_COUNT_MASK)
									| AVR32_USBA_END_TR_EN_MASK
									| AVR32_USBA_END_TR_IT_MASK
									| AVR32_USBA_END_B_EN_MASK
									| AVR32_USBA_END_BUFFIT_MASK
									| AVR32_USBA_CHANN_ENB_MASK );

    }
#endif

    return USBD_STATUS_SUCCESS;
}

//------------------------------------------------------------------------------
/// Put endpoint into Halt state
/// \param bEndpoint Index of endpoint
//------------------------------------------------------------------------------
void USBD_Halt( unsigned char bEndpoint )
{
    Endpoint *pEndpoint = &(endpoints[bEndpoint]);

    // Check that endpoint is enabled and not already in Halt state
    if( (pEndpoint->state != UDP_ENDPOINT_DISABLED)
     && (pEndpoint->state != UDP_ENDPOINT_HALTED) ) {

    	usart_write_line(USART1, "halt\n");

        // Abort the current transfer if necessary
        UDPHS_EndOfTransfer(bEndpoint, USBD_STATUS_ABORTED);

        // Put endpoint into Halt state
        usba->endpoint[bEndpoint].eptsetsta |= AVR32_USBA_FRCESTALL_MASK;
        //AT91C_BASE_UDPHS->UDPHS_EPT[bEndpoint].UDPHS_EPTSETSTA = AT91C_UDPHS_FRCESTALL;
        pEndpoint->state = UDP_ENDPOINT_HALTED;

#ifdef DMA
        // Test if endpoint type control
        if(AVR32_USBA_EPT_TYPE_CONTROL == (AVR32_USBA_EPT_TYPE_MASK & (usba->endpoint[bEndpoint].eptcfg))){
#endif
            // Enable the endpoint interrupt
            usba->ien |= (1 << AVR32_USBA_EPT_INT_0_OFFSET << bEndpoint);

#ifdef DMA
        }
        else {
            // Enable IT DMA
            usba->ien |= (1 << AVR32_USBA_DMA_INT_1_OFFSET << bEndpoint);
        }
#endif
   }
}

//------------------------------------------------------------------------------
/// Clears the Halt feature on the given endpoint.
/// \param bEndpoint Index of endpoint
//------------------------------------------------------------------------------
void USBD_Unhalt( unsigned char bEndpoint )
{
    Endpoint *pEndpoint = &(endpoints[bEndpoint]);

    // Check if the endpoint is enabled
    if (pEndpoint->state != UDP_ENDPOINT_DISABLED) {

        // Return endpoint to Idle state
        pEndpoint->state = UDP_ENDPOINT_IDLE;

        // Clear FORCESTALL flag
        usba->endpoint[bEndpoint].eptclrsta = AVR32_USBA_TOGGLESQ_MASK | AVR32_USBA_FRCESTALL_MASK;

        // Reset Endpoint Fifos
        usba->eptrst = (1 << bEndpoint);
    }
}

//------------------------------------------------------------------------------
/// Returns the current Halt status of an endpoint.
/// \param bEndpoint Index of endpoint
/// \return 1 if the endpoint is currently halted; otherwise 0
//------------------------------------------------------------------------------
unsigned char USBD_IsHalted( unsigned char bEndpoint )
{
    Endpoint *pEndpoint = &(endpoints[bEndpoint]);
    unsigned char status = 0;

    if (pEndpoint->state == UDP_ENDPOINT_HALTED) {
        status = 1;
    }
    return( status );
}

//------------------------------------------------------------------------------
/// IS High Speed device working in High Speed ?
/// \return 1 if the device is in High Speed; otherwise 0 (Full Speed)
//------------------------------------------------------------------------------
unsigned char USBD_IsHighSpeed( void )
{
    unsigned char status = 0;

    if ( AVR32_USBA_SPEED_MASK == (usba->intsta & AVR32_USBA_SPEED_MASK))
    {
        // High Speed
#ifdef DEBUG
    	usart_write_line(USART1, "High speed\n");
#endif
    	status = 1;
    }
    else {
    	usart_write_line(USART1, "Full speed\n");
    }
    return( status );
}

//------------------------------------------------------------------------------
/// Causes the endpoint to acknowledge the next received packet with a STALL
/// handshake.
/// Further packets are then handled normally.
/// \param bEndpoint Index of endpoint
/// \return Operation result code: USBD_STATUS_LOCKED or USBD_STATUS_SUCCESS
//------------------------------------------------------------------------------
unsigned char USBD_Stall( unsigned char bEndpoint )
{
    Endpoint *pEndpoint = &(endpoints[bEndpoint]);

    // Check that endpoint is in Idle state
    if (pEndpoint->state != UDP_ENDPOINT_IDLE) {
        return USBD_STATUS_LOCKED;
    }
#ifdef DEBUG
	usart_write_line(USART1, "Stalled\n");
#endif
    usba->endpoint[bEndpoint].eptsetsta = AVR32_USBA_FRCESTALL_MASK;

    return USBD_STATUS_SUCCESS;
}

//------------------------------------------------------------------------------
/// Activates a remote wakeup procedure
//------------------------------------------------------------------------------
void USBD_RemoteWakeUp(void)
{
    // Device is currently suspended
    if (deviceState == USBD_STATE_SUSPENDED) {

        UDPHS_EnableUsbClock();

        // Activates a remote wakeup
        usba->ctrl |= AVR32_USBA_REWAKEUP_MASK;

        while ((usba->ctrl & AVR32_USBA_REWAKEUP_MASK) == AVR32_USBA_REWAKEUP_MASK){};
        UDPHS_EnableBIAS();
    }
    // Device is NOT suspended
    else {
    	usart_write_line(USART1, "Remote wakeup\n");
    }
}

//------------------------------------------------------------------------------
/// Sets the device address
/// \param address Adress to be set
//------------------------------------------------------------------------------
void USBD_SetAddress( unsigned char address )
{
	usart_write_line(USART1, "Set address()\n");

    // Set address
    usba->ctrl &= ~AVR32_USBA_DEV_ADDR_MASK;

    usba->ctrl |= address | AVR32_USBA_FADDR_EN_MASK;

    // If the address is 0, the device returns to the Default state
    if (address == 0) {
        deviceState = USBD_STATE_DEFAULT;
        usart_write_line(USART1, "address is zero\n");
    }
    // If the address is non-zero, the device enters the Address state
    else {
        deviceState = USBD_STATE_ADDRESS;
        usart_write_line(USART1, "st - addressed\n");
    }
}

//------------------------------------------------------------------------------
/// Changes the device state from Address to Configured, or from Configured
/// to Address.
/// This method directly access the last received SETUP packet to decide on
/// what to do.
/// \param cfgnum configuration number
//------------------------------------------------------------------------------
void USBD_SetConfiguration( unsigned char cfgnum )
{
	usart_write_line(USART1, "Set configuration()\n");

    // Check the request
    if( cfgnum != 0 ) {

        // Enter Configured state
        deviceState = USBD_STATE_CONFIGURED;
    }
    // If the configuration number is zero, the device goes back to the Address
    // state
    else  {

        // Go back to Address state
        deviceState = USBD_STATE_ADDRESS;

        // Abort all transfers
        UDPHS_DisableEndpoints();
    }
}

//------------------------------------------------------------------------------
/// Enables the pull-up on the D+ line to connect the device to the USB.
//------------------------------------------------------------------------------
void USBD_Connect( void )
{
    usart_write_line(USART1, "Connect\n");
#if defined(BOARD_USB_PULLUP_INTERNAL)
    AT91C_BASE_UDPHS->UDPHS_CTRL &= ~AT91C_UDPHS_DETACH;   // Pull Up on DP
    AT91C_BASE_UDPHS->UDPHS_CTRL |= AT91C_UDPHS_PULLD_DIS; // Disable Pull Down

#elif defined(BOARD_USB_PULLUP_INTERNAL_BY_MATRIX)
    TRACE_DEBUG_WP("PUON 1\n\r");
    AT91C_BASE_MATRIX->MATRIX_USBPCR |= AT91C_MATRIX_USBPCR_PUON;

#elif defined(BOARD_USB_PULLUP_EXTERNAL)

#ifdef PIN_USB_PULLUP
    const Pin pinPullUp = PIN_USB_PULLUP;
    if( pinPullUp.attribute == PIO_OUTPUT_0 ) {

        PIO_Set(&pinPullUp);
    }
    else {

        PIO_Clear(&pinPullUp);
    }b
#else
    #error unsupported now
#endif

#elif !defined(BOARD_USB_PULLUP_ALWAYSON)
    usba->ctrl &= ~AVR32_USBA_DETACH_MASK; // EK
    //   #error Unsupported pull-up type.

#endif
}

//------------------------------------------------------------------------------
/// Disables the pull-up on the D+ line to disconnect the device from the bus.
//------------------------------------------------------------------------------
void USBD_Disconnect( void )
{
	usart_write_line(USART1, "Disconnect\n");

    // Device returns to the Powered state
    if (deviceState > USBD_STATE_POWERED) {
        deviceState = USBD_STATE_POWERED;
    }
}

//------------------------------------------------------------------------------
/// Certification test for High Speed device.
/// \param bIndex Test to be done
//------------------------------------------------------------------------------
void USBD_Test( unsigned char bIndex )
{

	usart_write_line(USART1, "Test()\n");

    char          *pFifo;
    unsigned char i;

    usba->ien &= ~AVR32_USBA_DET_SUSPD_MASK;
    //AT91C_BASE_UDPHS->UDPHS_IEN &= ~AT91C_UDPHS_DET_SUSPD; // remove suspend for TEST
    usba->tst |= AVR32_USBA_SPEED_CFG_FORCE_HIGH;
    //AT91C_BASE_UDPHS->UDPHS_TST |= AT91C_UDPHS_SPEED_CFG_HS; // force High Speed (remove suspend)

    switch( bIndex ) {

        case USBFeatureRequest_TESTPACKET:
          //  TRACE_DEBUG_WP("TEST_PACKET ");

            usba->dma[1].dmacontrol = 0;
            //AT91C_BASE_UDPHS->UDPHS_DMA[1].UDPHS_DMACONTROL = 0;
            usba->dma[2].dmacontrol = 0;
            //AT91C_BASE_UDPHS->UDPHS_DMA[2].UDPHS_DMACONTROL = 0;

            // Configure endpoint 2, 64 bytes, direction IN, type BULK, 1 bank
            usba->endpoint[2].eptcfg |= (AVR32_USBA_EPT_SIZE_SZ_64 << AVR32_USBA_EPT_SIZE_OFFSET)
										| AVR32_USBA_EPT_DIR_MASK
										| (AVR32_USBA_EPT_TYPE_BULK << AVR32_USBA_EPT_TYPE_OFFSET)
										| (AVR32_USBA_BK_NUMBER_ONE << AVR32_USBA_BK_NUMBER_OFFSET);
            //AT91C_BASE_UDPHS->UDPHS_EPT[2].UDPHS_EPTCFG = AT91C_UDPHS_EPT_SIZE_64 | AT91C_UDPHS_EPT_DIR_IN | AT91C_UDPHS_EPT_TYPE_BUL_EPT | AT91C_UDPHS_BK_NUMBER_1;

            while( (signed int)(usba->endpoint[2].eptcfg & AVR32_USBA_EPT_MAPD_MASK) != (signed int)AVR32_USBA_EPT_MAPD_MASK){}
            //while( (signed int)(AT91C_BASE_UDPHS->UDPHS_EPT[2].UDPHS_EPTCFG & AT91C_UDPHS_EPT_MAPD) != (signed int)AT91C_UDPHS_EPT_MAPD ) {}

            usba->endpoint[2].eptctlenb |= AVR32_USBA_EPT_ENABL_MASK;
            //AT91C_BASE_UDPHS->UDPHS_EPT[2].UDPHS_EPTCTLENB =  AT91C_UDPHS_EPT_ENABL;

            // Write FIFO
            pFifo = (char*)((unsigned int *)(AVR32_USBA_FIFO_ADDRESS) + (AVR32_USBA_FIFO_SIZE * 2));
            //pFifo = (char*)((unsigned int *)(AT91C_BASE_UDPHS_EPTFIFO->UDPHS_READEPT0) + (16384 * 2));
            for( i=0; i<sizeof(test_packet_buffer); i++) {
                pFifo[i] = test_packet_buffer[i];
            }
            // Tst PACKET
            usba->tst |= AVR32_USBA_TST_PKT;
            //AT91C_BASE_UDPHS->UDPHS_TST |= AT91C_UDPHS_TST_PKT;
            // Send packet
            usba->endpoint[2].eptsetsta |= AVR32_USBA_TX_PK_RDY_MASK;
            //AT91C_BASE_UDPHS->UDPHS_EPT[2].UDPHS_EPTSETSTA = AT91C_UDPHS_TX_PK_RDY;
            break;

        case USBFeatureRequest_TESTJ:
            //TRACE_DEBUG_WP("TEST_J ");
            usba->tst |= AVR32_USBA_TST_J_MASK;
            //AT91C_BASE_UDPHS->UDPHS_TST = AT91C_UDPHS_TST_J;
            break;

        case USBFeatureRequest_TESTK:
            //TRACE_DEBUG_WP("TEST_K ");
            usba->tst |= AVR32_USBA_TST_K_MASK;
        	//AT91C_BASE_UDPHS->UDPHS_TST = AT91C_UDPHS_TST_K;
            break;

        case USBFeatureRequest_TESTSE0NAK:
            //TRACE_DEBUG_WP("TEST_SEO_NAK ");
            usba->ien = 0;
        	//AT91C_BASE_UDPHS->UDPHS_IEN = 0;  // for test
            break;

        case USBFeatureRequest_TESTSENDZLP:
            //while( 0 != (AT91C_BASE_UDPHS->UDPHS_EPT[0].UDPHS_EPTSTA & AT91C_UDPHS_TX_PK_RDY ) ) {}

        	usba->endpoint[0].eptsetsta |= AVR32_USBA_TX_PK_RDY_MASK;
        	//AT91C_BASE_UDPHS->UDPHS_EPT[0].UDPHS_EPTSETSTA = AT91C_UDPHS_TX_PK_RDY;

        	//while( 0 != (AT91C_BASE_UDPHS->UDPHS_EPT[0].UDPHS_EPTSTA & AT91C_UDPHS_TX_PK_RDY ) ) {}
            //TRACE_DEBUG_WP("SEND_ZLP ");
            break;
    }

}


//------------------------------------------------------------------------------
/// Initializes the specified USB driver
/// This function initializes the current FIFO bank of endpoints,
/// configures the pull-up and VBus lines, disconnects the pull-up and
/// then trigger the Init callback.
//------------------------------------------------------------------------------
void USBD_Init( void )
{
    unsigned char i;

    usart_write_line(USART1, "USBA Init()\n");

    // Reset endpoint structures
    UDPHS_ResetEndpoints();

    // Enable and disable of the transceiver is automaticaly done by the IP.
    // Reset and enable IP UDPHS
    usba->ctrl &= ~AVR32_USBA_EN_USBA_MASK;
    usba->ctrl = AVR32_USBA_EN_USBA_MASK;

    // With OR without DMA !!!
    // Initialization of DMA

    for (i=1; i<= ((usba->ipfeatures & AVR32_USBA_DMA_CHANNEL_NBR_MASK) >> AVR32_USBA_DMA_CHANNEL_NBR_OFFSET); i++){

        // RESET endpoint canal DMA:
        // DMA stop channel command
    	usba->dma[i].dmacontrol = 0;

        // Disable endpoint
    	usba->endpoint[i].eptctldis = AVR32_USBA_SHRT_PCKT_MASK
									| AVR32_USBA_BUSY_BANK_MASK
									| AVR32_USBA_NAK_OUT_MASK
									| AVR32_USBA_NAK_IN_MASK
									| AVR32_USBA_STALL_SNT_MASK
									| AVR32_USBA_RX_SETUP_MASK
									| AVR32_USBA_TX_PK_RDY_MASK
									| AVR32_USBA_TX_COMPLT_MASK
									| AVR32_USBA_RX_BK_RDY_MASK
									| AVR32_USBA_ERR_OVFLW_MASK
									| AVR32_USBA_MDATA_RX_MASK
									| AVR32_USBA_DATAX_RX_MASK
									| AVR32_USBA_NYET_DIS_MASK
									| AVR32_USBA_INTDIS_DMA_MASK
									| AVR32_USBA_AUTO_VALID_MASK
									| AVR32_USBA_EPT_DISABL_MASK;


        // Clear status endpoint
        usba->endpoint[i].eptclrsta = AVR32_USBA_TOGGLESQ_MASK
									| AVR32_USBA_FRCESTALL_MASK
									| AVR32_USBA_RX_BK_RDY_MASK
									| AVR32_USBA_TX_COMPLT_MASK
									| AVR32_USBA_RX_SETUP_MASK
									| AVR32_USBA_STALL_SNT_MASK
									| AVR32_USBA_NAK_IN_MASK
									| AVR32_USBA_NAK_OUT_MASK;

        // Reset endpoint config
        usba->endpoint[i].eptctlenb = 0;

        // Reset DMA channel (Buff count and Control field)
		usba->dma[i].dmacontrol = AVR32_USBA_LDNXT_DSC_MASK;

        // Reset DMA channel 0 (STOP)
        usba->dma[i].dmacontrol = 0;

        // Clear DMA channel status (read the register for clear it)
        usba->dma[i].dmastatus = usba->dma[i].dmastatus;

    }

    usba->tst = 0;
    usba->ien = 0;
    usba->clrint = AVR32_USBA_UPSTR_RES_MASK
				 | AVR32_USBA_ENDOFRSM_MASK
				 | AVR32_USBA_WAKE_UP_MASK
				 | AVR32_USBA_ENDRESET_MASK
				 | AVR32_USBA_INT_SOF_MASK
				 | AVR32_USBA_MICRO_SOF_MASK
				 | AVR32_USBA_DET_SUSPD_MASK;

    // Device is in the Attached state
    deviceState = USBD_STATE_SUSPENDED;
    previousDeviceState = USBD_STATE_POWERED;


    // Enable interrupts
    usba->ien = AVR32_USBA_ENDOFRSM_MASK
		       | AVR32_USBA_WAKE_UP_MASK
		       | AVR32_USBA_DET_SUSPD_MASK;


    // Configure interrupts
    //USBDCallbacks_Initialized();

}

//------------------------------------------------------------------------------
/// Returns the current state of the USB device.
/// \return Device current state.
//------------------------------------------------------------------------------
unsigned char USBD_GetState( void )
{
    return deviceState;
}

//#endif // BOARD_USB_UDPHS

