/******************************************************************************
 * cdce706.c
 *
 *	\brief This file defines functions to control the external clock cdce706
 *
 * Cdce706 is a clock synthesizer used to clock the DAC and SSC1. It can be
 * programmed to generate all the clocks necessary for the desired audio sample
 * rates. The programming is perfomed over the TWI bus. An array of 29 bytes
 * is needed to specify all settings in the clock circuit. These settings
 * are stored in an array and sent to the clock when the frequency is to be
 * changed. All clock outputs are tristated when spdif mode is entered. The
 * TWI communication relies on the TWI driver provided by Atmel.
 *
 *  Created on: Feb 27, 2009
 *      Author: Espen Karlsen
 ******************************************************************************/

#include <avr32/io.h>
#include "cdce706.h"
#include "doxaUSB.h"
#include "twi.h"


// CDCE706 settings
// Input xtal 22.5792MHz, Sampling freq 44.1kHz
// More frequencies should be added
unsigned char cdce706_freq_list[1][29] = {
{0x00, 	27,		0x11,	0x01,	0x0C,	0x60,	0x08,	0x01,
0x80,	0x01,	0x01,	0x00,	0x00,	0x01,	0x02,	0x01,
0x01,	0x10,	0x08,	0x40,	0x01,	0x30,	0x30,	0x3A,
0x3B,	0x3C,	0x30,	0x00,	0x1B}
};


int cdce706_set_freq(volatile avr32_twi_t* twi, int freq)
{
#ifdef DEBUG_TWI
  usart_write_line(USART1, "Set freq\n");
#endif

  // Create a TWI package to send to the device.
  twi_package_t* cdce706_twi_package;

  cdce706_twi_package->addr = 0;               // No internal address
  cdce706_twi_package->addr_length = 0;        // Internal address length = 0
  cdce706_twi_package->buffer = &cdce706_freq_list[freq][0]; // Data buffer to send
  cdce706_twi_package->length = 29;            // Data buffer length
  cdce706_twi_package->chip = CDCE706_ADDRESS; // Chip address

  return twi_master_write(twi, cdce706_twi_package);
}


int cdce706_init(volatile avr32_twi_t* twi, twi_options_t* twi_opt)
{
  // Assign GPIO pins to TWI
  static const gpio_map_t TWI_GPIO_MAP =
    {
      {AVR32_TWI_SDA_0_PIN, AVR32_TWI_SDA_0_FUNCTION},
      {AVR32_TWI_SCL_0_PIN, AVR32_TWI_SCL_0_FUNCTION}
    };

  // TWI gpio pins configuration
  gpio_enable_module(TWI_GPIO_MAP, sizeof(TWI_GPIO_MAP) / sizeof(TWI_GPIO_MAP[0]));

  // Initialize twi master module
  return twi_master_init(twi, twi_opt);
}
