/******************************************************************************
 * cdce706.h
 *
 *	\brief This file defines functions to control the external clock cdce706
 *
 * Cdce706 is a clock synthesizer used to clock the DAC and SSC1. It can be
 * programmed to generate all the clocks necessary for the desired audio sample
 * rates. The programming is perfomed over the TWI bus. An array of 29 bytes
 * is needed to specify all settings in the clock circuit. These settings
 * are stored in an array and sent to the clock when the frequency is to be
 * changed. All clock outputs are tristated when spdif mode is entered. The
 * TWI communication relies on the TWI driver provided by Atmel.
 *
 *  Created on: Feb 27, 2009
 *      Author: Espen Karlsen
 ******************************************************************************/

#ifndef CDCE706_H_
#define CDCE706_H_

#include "twi.h"

#define BIT_CLOCK_PIN	CDCE706_Y1_PIN
#define WORD_CLOCK_PIN	CDCE706_Y2_PIN

#define CDCE706_XTAL_FREQ			30000000
#define CDCE706_SCL0CK_MAX_FREQ		100000
#define CDCE706_ADDRESS				0x69
#define CDCE706_READ				0
#define CDCE706_WRITE				1
#define CDCE706_CMD_BYTE_MASK		0b10000000
#define CDCE706_CMD_BLOCK_MASK		0b00000000

#define CDCE706_REVISION_CODE			0
#define CDCE706_REVISION_CODE_OFFSET 	4
#define CDCE706_REVISION_CODE_MASK		0xf0
#define CDCE706_VENDOR_ID				1
#define CDCE706_VENDOR_ID_OFFSET		0
#define CDCE706_VENDOR_ID_MASK			0x0f

#define CDCE706_PLL_REF_DIVIDER_M	1
#define CDCE706_PLL_REF_DIVIDER_N	2
#define CDCE706_PLL1_MUX_OFFSET		7
#define CDCE706_PLL2_MUX_OFFSET		6
#define CDCE706_PLL3_MUX_OFFSET		5

#define CDCE706_PLL1_FVCO_OFFSET	7
#define CDCE706_PLL2_FVCO_OFFSET	6
#define CDCE706_PLL3_FVCO_OFFSET	5

#define CDCE706_PLL_SEL_P0_OFFSET		5
#define CDCE706_PLL_SEL_P1_OFFSET		5
#define CDCE706_PLL_SEL_P2_OFFSET		0
#define CDCE706_PLL_SEL_P3_OFFSET		3
#define CDCE706_PLL_SEL_P4_OFFSET		0
#define CDCE706_PLL_SEL_P5_OFFSET		3

#define CDCE706_CKSEL_OFFSET			4
#define CDCE706_CONFIG_INP_S0_OFFSET	0
#define CDCE706_CONFIG_INP_S0_MASK		0x03
#define CDCE706_CONFIG_INP_S1_OFFSET	2
#define CDCE706_CONFIG_INP_S1_MASK		0x0c

#define CDCE706_INP_SIG_SOURCE_OFFSET	6
#define CDCE706_POWER_DOWN_OFFSET		6

#define CDCE706_DIVIDER_PO_ADR			13
#define CDCE706_DIVIDER_P1_ADR			14
#define CDCE706_DIVIDER_P2_ADR			15
#define CDCE706_DIVIDER_P3_ADR			16
#define CDCE706_DIVIDER_P4_ADR			17
#define CDCE706_DIVIDER_P5_ADR			18

#define CDCE706_INV_OFFSET				6
#define CDCE706_Y0_INV_OFFSET			6
#define CDCE706_Y1_INV_OFFSET			6
#define CDCE706_Y2_INV_OFFSET			6
#define CDCE706_Y3_INV_OFFSET			6
#define CDCE706_Y4_INV_OFFSET			6
#define CDCE706_Y5_INV_OFFSET			6
#define CDCE706_Y0_INV_MASK				0x40
#define CDCE706_Y1_INV_MASK				0x40
#define CDCE706_Y2_INV_MASK				0x40
#define CDCE706_Y3_INV_MASK				0x40
#define CDCE706_Y4_INV_MASK				0x40
#define CDCE706_Y5_INV_MASK				0x40

#define CDCE706_SR_CONTROL_OFFSET		4
#define CDCE706_Y0_SR_CONTROL_OFFSET	4
#define CDCE706_Y1_SR_CONTROL_OFFSET	4
#define CDCE706_Y2_SR_CONTROL_OFFSET	4
#define CDCE706_Y3_SR_CONTROL_OFFSET	4
#define CDCE706_Y4_SR_CONTROL_OFFSET	4
#define CDCE706_Y5_SR_CONTROL_OFFSET	4

#define CDCE706_ENABLE_OFFSET			3
#define CDCE706_Y0_ENABLE_OFFSET		3
#define CDCE706_Y1_ENABLE_OFFSET		3
#define CDCE706_Y2_ENABLE_OFFSET		3
#define CDCE706_Y3_ENABLE_OFFSET		3
#define CDCE706_Y4_ENABLE_OFFSET		3
#define CDCE706_Y5_ENABLE_OFFSET		3
#define CDCE706_Y0_ENABLE_MASK			0x08
#define CDCE706_Y1_ENABLE_MASK			0x08
#define CDCE706_Y2_ENABLE_MASK			0x08
#define CDCE706_Y3_ENABLE_MASK			0x08
#define CDCE706_Y4_ENABLE_MASK			0x08
#define CDCE706_Y5_ENABLE_MASK			0x08

#define CDCE706_DIV_SEL_OFFSET			0
#define CDCE706_Y0_DIV_SEL_OFFSET		0
#define CDCE706_Y1_DIV_SEL_OFFSET		0
#define CDCE706_Y2_DIV_SEL_OFFSET		0
#define CDCE706_Y3_DIV_SEL_OFFSET		0
#define CDCE706_Y4_DIV_SEL_OFFSET		0
#define CDCE706_Y5_DIV_SEL_OFFSET		0

#define CDCE706_SSC_MOD_SEL_OFFSET		4
#define CDCE706_SSC_FREQ_SEL_OFFSET		0
#define CDCE706_EELOCK_OFFSET			7
#define CDCE706_EEPIP_OFFSET			7
#define CDCE706_EEWRITE_OFFSET			7

#define CDCE706_BYTE_COUNT_OFFSET		0


/*! \brief Sets up registers and twi module as master
 *
 *  \param twi Pointer to the twi module address
 *  \param twi_opt Paramters to initialize twi module
 *  \return Status
 * 	\return TWI master write status
 * 		\retval TWI_INVALID_ARGUMENT
 * 		\retval TWI_RECEIVE_NACK
 * 		\retval TWI_SUCCESS
 */
int cdce706_init(volatile avr32_twi_t* twi, twi_options_t* twi_opt);


/** \brief Function to alter the output frequencies of the clock circuit
 *
 * Program the CDCE706 clock generator with a pre-calculated byte
 * array containg the necessary configuration for the given sampling
 * rate.
 *
 * \param int freq sampling frequency as defined by \c SAMPLING_FREQ_44_1KHZ etc.
 * \return TWI master write status
 * 		\retval TWI_INVALID_ARGUMENT
 * 		\retval TWI_RECEIVE_NACK
 * 		\retval TWI_SUCCESS
 */
int cdce706_set_freq(volatile avr32_twi_t* twi, int freq);

/* CDCE706_H_ */
#endif
