/******************************************************************************
 * dir9001.c
 *
 *	\brief This file defines a set of functions to control the dir9001 circuit.
 *
 *	The dir9001 ciruit is a SPDIF receiver which receives spdif data and
 *	generates dac clock signals recovered from an input spdif signal. When the
 *	system is in SPDIF mode, the dir9001 are connected to the DACs clock lines
 *	and it drives the DAC. Also an spdif signal conditioner is enabled in this
 *	mode.
 *
 *  Created on: Feb 17, 2009
 *      Author: Espen Karlsen
 ******************************************************************************/


#include "doxaUSB.h"
#include "dir9001.h"
#include "pcm1794a.h"

// Memory pointer to pioa module
volatile avr32_pio_t *dir9001 = &AVR32_PIOA;

void dir9001_init(void){

	// GPIO enable
	dir9001->per  |= 1 << SPDIF_FMT0_PIN
			  | 1 << SPDIF_FMT1_PIN
			  | 1 << SPDIF_RST_PIN
			  | 1 << SPDIF_AUDIO_PIN
			  | 1 << SPDIF_ERROR_PIN
			  | 1 << SPDIF_BFRAME_PIN
			  | 1 << SPDIF_EMPH_PIN
			  | 1 << SPDIF_UOUT_PIN
			  | 1 << SPDIF_COUT_PIN
			  | 1 << SPDIF_PSCK0_PIN
			  | 1 << SPDIF_PSCK1_PIN
			  | 1 << SPDIF_EN_PIN;

	// Pull-up disable
	dir9001->pudr |= 1 << SPDIF_FMT0_PIN
			  | 1 << SPDIF_FMT1_PIN
			  | 1 << SPDIF_RST_PIN
			  | 1 << SPDIF_AUDIO_PIN
			  | 1 << SPDIF_ERROR_PIN
			  | 1 << SPDIF_BFRAME_PIN
			  | 1 << SPDIF_EMPH_PIN
			  | 1 << SPDIF_UOUT_PIN
			  | 1 << SPDIF_COUT_PIN
			  | 1 << SPDIF_PSCK0_PIN
			  | 1 << SPDIF_PSCK1_PIN
			  | 1 << SPDIF_EN_PIN;

	// Output enable
	dir9001->oer  |= 1 << SPDIF_FMT0_PIN
			  | 1 << SPDIF_FMT1_PIN
			  | 1 << SPDIF_RST_PIN
			  | 1 << SPDIF_PSCK0_PIN
			  | 1 << SPDIF_PSCK1_PIN
			  | 1 << SPDIF_EN_PIN;

	// Output write enable
	dir9001->ower |= 1 << SPDIF_FMT0_PIN
			  | 1 << SPDIF_FMT1_PIN
			  | 1 << SPDIF_RST_PIN
			  | 1 << SPDIF_PSCK0_PIN
			  | 1 << SPDIF_PSCK1_PIN
			  | 1 << SPDIF_EN_PIN;

	// Input glitch filter enable
	dir9001->ifer |= 1 <<SPDIF_AUDIO_PIN
			  | 1 << SPDIF_ERROR_PIN
			  | 1 << SPDIF_BFRAME_PIN
			  | 1 << SPDIF_EMPH_PIN
			  | 1 << SPDIF_UOUT_PIN
			  | 1 << SPDIF_COUT_PIN;


	// Default 16-bit MSB first, rigth justified, reset active
	dir9001_reset_active();
	dir9001_set_format(RIGHT_JUSTIFIED_16_BIT);
	dir9001_set_freq(SCKO_256_FS);
}

int dir9001_set_format(int format){

	// Set output pins according to given format setting
	switch(format){
	case RIGHT_JUSTIFIED_16_BIT:
		dir9001->odsr &= ~(1 << SPDIF_FMT1_PIN | 1 << SPDIF_FMT0_PIN);
		break;
	case RIGHT_JUSTIFIED_24_BIT:
		dir9001->odsr = (dir9001->odsr & ~(1 << SPDIF_FMT1_PIN)) | (1 << SPDIF_FMT0_PIN);
		break;
	case LEFT_JUSTIFIED_24_BIT:
		dir9001->odsr = (1 << SPDIF_FMT1_PIN) | (dir9001->odsr & ~(1 << SPDIF_FMT0_PIN));
		break;
	case I2S_24_BIT:
		dir9001->odsr |= (1 << SPDIF_FMT1_PIN) | (1 << SPDIF_FMT0_PIN);
		break;
	default:
		return -1;	// Invalid format
	}
	return 0;
}

int dir9001_set_freq(int freq){

	// Set output pins according to given frequency setting
	switch(freq){
	case SCKO_128_FS:
		dir9001->odsr &= ~(1 << SPDIF_PSCK1_PIN | 1 << SPDIF_PSCK0_PIN);
		break;
	case SCKO_256_FS:
		dir9001->odsr = (dir9001->odsr & ~(1 << SPDIF_PSCK1_PIN)) | (1 << SPDIF_PSCK0_PIN);
		break;
	case SCKO_384_FS:
		dir9001->odsr = (1 << SPDIF_PSCK1_PIN) | (dir9001->odsr & ~(1 << SPDIF_PSCK0_PIN));
		break;
	case SCKO_512_FS:
		dir9001->odsr |= (1 << SPDIF_PSCK1_PIN) | (1 << SPDIF_PSCK0_PIN);
		break;
	default:
		return -1;	// Invalid frequency
	}
	return 0;

}

void dir9001_reset_active(void){
	// Clear pin
	dir9001->odsr &= ~(1 << SPDIF_RST_PIN);
}

void dir9001_reset_deactive(void){
	// Set pin
	dir9001->odsr |= (1 << SPDIF_RST_PIN);
}

void dir9001_enable(void){
	// Clear pin
	dir9001->odsr &= ~(1 << SPDIF_EN_PIN);
}

void dir9001_disable(void){
	// Set pin
	dir9001->odsr |= (1 << SPDIF_EN_PIN);
}
