/******************************************************************************
 * dir9001.h
 *
 *	\brief This file defines a set of functions to control the dir9001 circuit.
 *
 *	The dir9001 ciruit is a SPDIF receiver which receives spdif data and
 *	generates dac clock signals recovered from an input spdif signal. When the
 *	system is in SPDIF mode, the dir9001 are connected to the DACs clock lines
 *	and it drives the DAC. Also an spdif signal conditioner is enabled in this
 *	mode.
 *
 *  Created on: Feb 17, 2009
 *      Author: Espen Karlsen
 ******************************************************************************/

#ifndef DIR9001_H
#define DIR9001_H

#include <avr32/io.h>
#include "doxaUSB.h"

// System clock settings
#define	SCKO_128_FS		0
#define SCKO_256_FS		1
#define SCKO_384_FS		2
#define SCKO_512_FS		3


/*! \brief Initialize io pins for dir9001
 *
 * Sets up io control pins for dir9001. Default setting is right justified 16
 * bit data, system clock = 256 fs and reset active.
 */
void dir9001_init(void);

/*! \brief Controls the data format of the dir9001 circuit.
 * \param format
 * 		\arg RIGHT_JUSTIFIED_16_BIT
 *		\arg RIGHT_JUSTIFIED_24_BIT
 *		\arg LEFT_JUSTIFIED_24_BIT
 *	 	\arg I2S_24_BIT
 */
int dir9001_set_format(int format);

/*! \brief Controls the system clock frequency
 * 	\param freq
 * 		\arg SCKO_128_FS
 * 		\arg SCKO_256_FS
 * 		\arg SCKO_384_FS
 * 		\arg SCKO_512_FS
 */
int dir9001_set_freq(int freq);

/*! \brief Enable dir9001 SPDIF receiver and bus access
 * Pulls the SPDIF_EN_PIN high
 */
void dir9001_enable(void);

/*! \brief Disable dir9001 SPDIF receiver and bus access
 * Pulls the SPDIF_EN_PIN low
 */
void dir9001_disable(void);

/*! \brief Enable reset on dir9001 SPDIF receiver
 * Pulls the reset pin low
 */
void dir9001_reset_active(void);

/*! \brief Disable reset on dir9001 SPDIF receiver
 * Pulls the reset pin high
 */
void dir9001_reset_deactive(void);

/* DIR9001_H */
#endif
