/******************************************************************************
 * doxaUSB.c
 *
 *	\brief Master source file of the doxaUSB project.
 *
 * This project implements a usb audio streaming device and a spdif
 * receiver/decoder. Control signals for an external clock and dac are generated.
 * When data are received from the USB audio stream, they are read out and
 * buffered continously in a loop, independent of the audio input source selected.
 * The buttons used to select volume and input source are interrupt driven so
 * the source selection and volume are controlled independent of the loop.
 *
 *	Compiler:           GNU GCC for AVR32
 * 	Supported devices:  AP7000/AP7001/AP7002
 *
 *  Created on: Feb 2, 2009
 *      Author: Espen Karlsen
 ******************************************************************************/

#include <avr32/io.h>
#include "nlao_interrupts.h"
#include "doxaUSB.h"
#include "compiler.h"
#include "board.h"
#include "pm_at32ap7000.h"
#include "gpio.h"
#include "usart.h"
#include "intc.h"
#include "twi.h"
#include "dir9001.h"
#include "pcm1794a.h"
#include "cdce706.h"
#include "ui.h"
#include "volumecontrol.h"
#include "USBD.h"
#include "USBGenericRequest.h"
#include "USBDDriver.h"
#include "USBDDriverDescriptors.h"
#include "USBDCallbacks.h"
#include "AUDIODriverDescriptors.h"
#include "AUDIODriver.h"


//------------------------------------------------------------------------------
//         Internal variables
//------------------------------------------------------------------------------
/// State of USB, for suspend and resume
unsigned char USBState = STATE_IDLE;

/// USB standard device driver.
USBDDriver usbdDriver;

// Memory pointers
volatile avr32_pm_t 	*pm = &AVR32_PM;
volatile avr32_usba_t 	*usba = &AVR32_USBA;
volatile avr32_twi_t 	*twi = &AVR32_TWI;
volatile avr32_usart_t 	*usart1 = &AVR32_USART1;

/// DAC variable
extern pcm1794a_s pcm1794a;

/// Buffers for USB audio
/// Data buffers for receiving audio frames from the USB host
volatile unsigned char aud_buf[AUDIO_BUFFER_COUNT][AUDIODriver_BYTESPERFRAME];

/// Number of samples stored in each data buffer.
volatile unsigned short aud_buf_sizes[AUDIO_BUFFER_COUNT];

/// Next buffer which should be sent to the DAC.
volatile unsigned int usb_buf_indx = 0;

/// Number of buffers that can be sent to the DAC.
volatile unsigned int aud_buf_rdy = 0;

/// Variable to indicate if dac is active
int isDacActive = 0;

/// Number of buffers to be filled before DAC starts playing
#define DAC_DELAY  	5


//------------------------------------------------------------------------------
//         Callbacks re-implementation
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
/// Invoked when the USB device leaves the Suspended state.
//------------------------------------------------------------------------------
void USBDCallbacks_Resumed(void)
{
#ifdef DEBUG
  usart_write_line(USART1, "Resumed callback\n");
#endif
  USBState = STATE_RESUME;
}

//------------------------------------------------------------------------------
/// Invoked when the USB device gets suspended.
//------------------------------------------------------------------------------
void USBDCallbacks_Suspended(void)
{
#ifdef DEBUG
  usart_write_line(USART1, "Suspended callback\n");
#endif
  USBState = STATE_SUSPEND;
}


/*! \brief Callback called when a frame has been received on USB
 * \param unused
 * \param status Read status
 * \param transferred Number of bytes received
 * \param remaining	Number of bytes remaining to receive
 */
static void FrameReceived(unsigned int unused,
                          char status,
                          unsigned int transferred,
                          unsigned int remaining)
{
#ifdef DEBUG
  usart_write_line(USART1, "FR \n");
#endif

  // Number of samples in buffer
  aud_buf_sizes[usb_buf_indx] = transferred / AUDIODriver_BYTESPERSAMPLE;

  // Switch to next buffer
  usb_buf_indx = (usb_buf_indx + 1) % AUDIO_BUFFER_COUNT;

  // One more buffer filled
  aud_buf_rdy++;

  if(aud_buf_rdy > AUDIO_BUFFER_COUNT)
	  aud_buf_rdy = AUDIO_BUFFER_COUNT;

  if(!isDacActive){
	  if(aud_buf_rdy > DAC_DELAY){
	  isDacActive = 1;
		  pcm1794a_init();
		  pcm1794a_write();
	  }
  }

  // Start new USB readout
  USBD_Read(AUDIODriverDescriptors_DATAOUT,		// Data out endpoint
                   &aud_buf[usb_buf_indx][0],	// Data out buffer
                   AUDIODriver_BYTESPERFRAME,	// Length of data out buffer
                   FrameReceived,				// Callback function after read
                   0);
}


/*! \brief Interrupt handler for USART module
 *
 */
//__attribute__((__interrupt__)) static void usart_int_handler(void)
//{
//  int c;
//  usart_read_char(USART1, &c);
//  usart_write_char(USART1, c);
//}


/*! \brief Initializes the power manager (PM).
 *
 * TODO: Disable clocks for all unused modules in the clock mask register.
 */
int pm_init(void)
{
  // Reset PM. Makes sure we get the expected clocking after a soft reset (e.g.: JTAG reset).
  pm_reset();

  // Start PLL0, Freq = OSC0 * 6 = 120 MHz, wait 10 slow RC cycles before lock flag is set.
  pm->pll0 = ((5 << AVR32_PM_PLLMUL_OFFSET)
	       | (10 << AVR32_PM_PLLCOUNT_OFFSET)
	       | (4 << AVR32_PM_PLLOPT_OFFSET)
	       | (1 << AVR32_PM_PLLEN_OFFSET))
	       | (0 << AVR32_PM_PLLOSC_OFFSET);

  // Clock frequency scaling, must have Fcpu >= Fhsb >= Fpba,b.
  pm->cksel = ((1 << AVR32_PM_PBADIV_OFFSET) | (1 << AVR32_PM_PBASEL_OFFSET) |
	       (1 << AVR32_PM_PBBDIV_OFFSET) | (0 << AVR32_PM_PBBSEL_OFFSET) |
	       (1 << AVR32_PM_HSBDIV_OFFSET) | (0 << AVR32_PM_HSBSEL_OFFSET) |
	       (0 << AVR32_PM_CPUDIV_OFFSET) | (0 << AVR32_PM_CPUSEL_OFFSET));

  // Wait for PLL0 to lock.
  while(!(pm->isr & AVR32_PM_LOCK0_MASK));

  // Clear PLL0 lock flag.
  pm->icr &= (~AVR32_PM_LOCK0_MASK);

   //PLL0 is the source for main clock, must be set after enabling PLL0.
  pm->mcctrl = (1 << AVR32_PM_PLLSEL_OFFSET);

  return 0;
}


/** \brief Main function
 *
 * Starts by initializing the required hardware modules and their
 * interrupts.
 */
int main(void)
{

  // Initialize system clocks
  pm_init();

  // Assign GPIO to USART.
  gpio_enable_module(USART_GPIO_MAP, sizeof(USART_GPIO_MAP) / sizeof(USART_GPIO_MAP[0]));

  // Initialize USART in RS232 mode.
  usart_init_rs232(USART1, &USART_OPTIONS, pm_get_clock_source_freq_hz(PM_PLL0) \
		  / pm_read_clock_domain_scaler(PM_PBA_DOMAIN));

  Disable_global_interrupt();

  // Initialize interrupt vectors.
  INTC_init_interrupts();

  // Register the USART interrupt handler to the interrupt controller.
  //INTC_register_interrupt(&usart_int_handler, AVR32_USART1_IRQ, AVR32_INTC_INT0);

  // Startup message
  usart_write_line(USART1, "\n\n¤¤¤ Doxa Preamp DAC ¤¤¤\n");

  // Initialize usba --------------------------------------------------------
  AUDDSpeakerDriver_Initialize();
  // Register the USBA interrupt handler to the interrupt controller.
  INTC_register_interrupt(&USBD_InterruptHandler, AVR32_USBA_IRQ, AVR32_INTC_INT1);
  //-------------------------------------------------------------------------

  // Initialize DAC ---------------------------------------------------------
  pcm1794a.dem  = 0;	// No de-emphasis
  pcm1794a.mute = 0;	// Mute off
  pcm1794a.rst  = 1;	// Reset deactive
  pcm1794a.fmt0 = 0;	// 16 bit standard stereo
  pcm1794a.fmt1 = 1;
  pcm1794a.chsl = 0;
  pcm1794a.mono = 0;
  pcm1794a_init();
  pcm1794a_write();
  //-------------------------------------------------------------------------

  // Initialize volume control ----------------------------------------------
  set_mute(MUTE_OFF);
  set_volume(LEFT_VOL, 18);
  set_volume(RIGHT_VOL, 18);
  set_source(SPDIF);
  init_volume_ctrl();
  //-------------------------------------------------------------------------

  // Initialize buttons -----------------------------------------------------
  init_buttons();
  //-------------------------------------------------------------------------

  // Initialize SPDIF -------------------------------------------------------
  dir9001_init();
  dir9001_reset_deactive();
  //-------------------------------------------------------------------------

  Enable_global_interrupt();


  // Initialize TWI ---------------------------------------------------------
  twi_options_t *twi_opt;
  twi_opt->chip = 0xaa;		// Master address
  twi_opt->pba_hz = pm_get_clock_source_freq_hz(PM_PLL0) \
  / pm_read_clock_domain_scaler(PM_PBA_DOMAIN);
  twi_opt->speed = 100000;	// Baud rate

  // Set twi parameters
  int twi_status = cdce706_init(twi,twi_opt);
#ifdef DEBUG_TWI
  if(twi_status == SUCCESS){
	  usart_write_line(USART1, "TWI init ok\n");
  }else{
	  usart_write_line(USART1, "TWI init failed\n");
  }
#endif

  // Set sampling freq to 44.1 default
  twi_status = cdce706_set_freq(twi,SAMPLING_FREQ_44_1KHZ);
#ifdef DEBUG_TWI
  if(twi_status == SUCCESS){
	  usart_write_line(USART1, "TWI set freq ok");
	}else{
	  usart_write_line(USART1, "TWI set freq failed");
	}
#endif

  //-------------------------------------------------------------------------

  // Wait for USB to get configured
  while (USBD_GetState() < USBD_STATE_CONFIGURED);

  // Infinite loop
  for(;;){

	  if (USBState == STATE_SUSPEND){
		  usart_write_line(USART1, "USB suspended\n");
		  USBState = STATE_IDLE;
	  }
      if (USBState == STATE_RESUME){
    	  usart_write_line(USART1, "USB resumed\n");
    	  USBState = STATE_IDLE;
      }

      // Read USB data
      USBD_Read(AUDIODriverDescriptors_DATAOUT,		// Data out endpoint
                        &aud_buf[usb_buf_indx][0],	// Data out buffer
                        AUDIODriver_BYTESPERFRAME,	// Length of data out buffer
                        FrameReceived,				// Callback function after read
                        0);
  }//for
}
