/******************************************************************************
 * doxaUSB.c
 *
 *	\brief Master header file of the doxaUSB project.
 *
 * This project implements a usb audio streaming device and a spdif
 * receiver/decoder. Control signals for an external clock and dac are generated.
 * When data are received from the USB audio stream, they are read out and
 * buffered continously in a loop, independent of the audio input source selected.
 * The buttons used to select volume and input source are interrupt driven so
 * the source selection and volume are controlled independent of the loop.
 *
 *	Compiler:           GNU GCC for AVR32
 * 	Supported devices:  AP7000/AP7001/AP7002
 *
 *  Created on: Feb 2, 2009
 *      Author: Espen Karlsen
 ******************************************************************************/

#ifndef DOXAUSB_H_
#define DOXAUSB_H_

#include <avr32/io.h>
#include "gpio.h"
#include "usart.h"
#include "pm_at32ap7000.h"
#include "volumecontrol.h"

//#define DEBUG
//#define DEBUG_TWI
//#define DEBUG_DAC

/// Clock domain frequencies
//!  @{
#define MAIN_FREQ	AVR32_PM_CPU_MAX_FREQ
#define HSB_FREQ	AVR32_PM_HSB_MAX_FREQ
#define PBA_FREQ	AVR32_PM_PBA_MAX_FREQ
#define PBB_FREQ	AVR32_PM_PBB_MAX_FREQ
//! @}

/// USART settings
//! @{
#  define USART1               (&AVR32_USART2)
#  define USART1_RX_PIN        AVR32_USART2_RXD_0_PIN
#  define USART1_RX_FUNCTION   AVR32_USART2_RXD_0_FUNCTION
#  define USART1_TX_PIN        AVR32_USART2_TXD_0_PIN
#  define USART1_TX_FUNCTION   AVR32_USART2_TXD_0_FUNCTION
//! @}


/// Chip has a UDPHS controller.
#define BOARD_USB_UDPHS

/// Number of endpoints in the USB controller.
#define BOARD_USB_NUMENDPOINTS                  7

/// Returns the maximum packet size of the given endpoint.
#define BOARD_USB_ENDPOINTS_MAXPACKETSIZE(i)    ((i == 0) ? 64 : (i == 1) ? 512 : 1024)

/// Returns the number of FIFO banks for the given endpoint.
#define BOARD_USB_ENDPOINTS_BANKS(i)            ((i == 0) ? 1 : ((i == 1) || (i == 2)) ? 2 : 3)

// USB configuration: Selfpowered, remote wakeup
#define BOARD_USB_BMATTRIBUTES	USBConfigurationDescriptor_SELFPOWERED_RWAKEUP

/// Use for power management
#define STATE_IDLE    0
/// The USB device is in suspend state
#define STATE_SUSPEND 4
/// The USB device is in resume state
#define STATE_RESUME  5

/// Audio data formats
//!  @{
#define RIGHT_JUSTIFIED_16_BIT	0
#define RIGHT_JUSTIFIED_24_BIT 	1
#define LEFT_JUSTIFIED_24_BIT	2
#define I2S_24_BIT				3
//! @}

/// Audio sampling frequencies
//! @{
#define SAMPLING_FREQ_32KHZ 	0
#define SAMPLING_FREQ_44_1KHZ 	0
#define SAMPLING_FREQ_48KHZ 	2
#define SAMPLING_FREQ_96KHZ 	3
#define SAMPLING_FREQ_192KHZ 	4
//! @}

/// PIO mapping of DAC control pins
//! @{
#define PCM1794A_MONO_PIN		11
#define PCM1794A_CHSL_PIN		12
#define PCM1794A_DEM_PIN		13
#define PCM1794A_MUTE_PIN		10
#define PCM1794A_FMT0_PIN		9
#define PCM1794A_FMT1_PIN		8
#define PCM1794A_ZERO_PIN		5
#define PCM1794A_RST_PIN		3
//! @}

/// PIO mapping of SPDIF decoder control pins
//! @{
#define SPDIF_AUDIO_PIN 	14
#define SPDIF_EMPH_PIN		21
#define SPDIF_UOUT_PIN		20
#define SPDIF_COUT_PIN		19
#define SPDIF_BFRAME_PIN	22
#define SPDIF_RST_PIN		24
#define SPDIF_FMT0_PIN		23
#define SPDIF_FMT1_PIN		17
#define SPDIF_ERROR_PIN		18
#define SPDIF_PSCK0_PIN		15
#define SPDIF_PSCK1_PIN		16
#define SPDIF_RXIN_PIN		2	// HW connector not PIO
#define SPDIF_EN_PIN		25
//! }@

/// PIO mapping of volume control
//! @{
#define VOLUME_LATCH_PIN 			0
#define VOLUME_UP_BUTTON_PIN		17
#define VOLUME_DOWN_BUTTON_PIN		16
#define VOLUME_MUTE_LED_PIN			11
#define SOURCE_CHANGE_BUTTON_PIN	14
#define VOLUME_MUTE_BUTTON_PIN		15
#define SOURCE_USB_LED_PIN			12
#define SOURCE_SPDIF_LED_PIN		13
//! }@

/// SPDIF output enable pin, PORTA
#define SPDIF_OE_PIN				25

/// USART gpio mapping
static const gpio_map_t USART_GPIO_MAP =
  {
    {USART1_RX_PIN, USART1_RX_FUNCTION},
    {USART1_TX_PIN, USART1_TX_FUNCTION}
  };

/// USART options
static const usart_options_t USART_OPTIONS =
  {
    .baudrate     = 115200,
    .charlength   = 8,
    .paritytype   = USART_NO_PARITY,
    .stopbits     = USART_1_STOPBIT,
    .channelmode  = USART_NORMAL_CHMODE
  };

/// Sample rate in Hz.
#define AUDIODriver_SAMPLERATE       	44100
/// Minimum supported sampling rate
#define AUDIODriver_SAMPLERATE_MIN		32000
/// Maximum supported sampling rate
#define AUDIODriver_SAMPLERATE_MAX		192000
/// Number of audio channels
#define AUDIODriver_NUMCHANNELS       2
/// Number of bytes in one sample.
#define AUDIODriver_BYTESPERSAMPLE    2
/// Number of bits in one sample.
#define AUDIODriver_BITSPERSAMPLE     (AUDIODriver_BYTESPERSAMPLE * 8)
/// Number of bytes in one USB subframe.
#define AUDIODriver_BYTESPERSUBFRAME  (AUDIODriver_NUMCHANNELS * \
                                            AUDIODriver_BYTESPERSAMPLE)
/// Number of samples in one USB frame.
#define AUDIODriver_SAMPLESPERFRAME   (AUDIODriver_SAMPLERATE / 800 \
                                            * AUDIODriver_NUMCHANNELS)
/// Number of bytes in one USB frame.
#define AUDIODriver_BYTESPERFRAME     (AUDIODriver_SAMPLESPERFRAME * \
                                            AUDIODriver_BYTESPERSAMPLE)
// Number of audio data buffers
#define 	AUDIO_BUFFER_COUNT	50


/// Buffers for USB audio

/// Data buffers to store audio data from the USB host
extern volatile unsigned char aud_buf[AUDIO_BUFFER_COUNT][AUDIODriver_BYTESPERFRAME];

/// Number of samples stored in each data buffer.
extern volatile unsigned short aud_buf_sizes[AUDIO_BUFFER_COUNT];

/// Next buffer which should be sent to the DAC.
extern volatile unsigned int usb_buf_indx;

/// Number of buffers that can be sent to the DAC.
extern volatile unsigned int aud_buf_rdy;

/// Volume control variable
extern volume_ctrl_t volume_ctrl;


#endif  /* DOXAUSB_H_ */
