/******************************************************************************
 * pcm1794a.h
 *
 *	\brief This file defines functions to control the pcm1794a.
 *
 *	PCM1794a is the dataconverter implemented in the system. This receives data
 *	from either the dir9001 SPDIF receiver or from the USB connection via the
 *	SSC1 module. Clocking is performed from the dir9001 in SPDIF mode and from
 *	the cdce706 external clock in USB mode. Since both the dir9001 and the
 *	SSC1 module/clock is connected to the same clock and data lines, they cannot
 *	be active simulaneously. When the system is in SPDIF mode the SSC1 must be
 *	deactivated to avoid drive conflict. In USB mode is audio data clocked from
 *	SSC1 to the DAC by the external clock and the signal from dir9001 is
 *	tristated. The control pins of the DAC is driven by general purpose io pins.
 *
 *  Created on: Mar 1, 2009
 *      Author: Espen Karlsen
 ******************************************************************************/

#include <avr32/io.h>
#include "pcm1794a.h"
#include "intc.h"
#include "doxaUSB.h"
#include "usart.h"

// Index in current samples buffer
static unsigned int dac_samples_indx = 0;
// Current buffer number
static unsigned int dac_buf_indx = 0;
// Current usb buffer number
extern volatile unsigned int usb_buf_indx;

// Dac settings
pcm1794a_s pcm1794a;

volatile avr32_pio_t *pcm1794a_pio = &AVR32_PIOA;
volatile avr32_ssc_t *pcm1794a_ssc_pio = &AVR32_SSC1;


void pcm1794a_init(void){

#ifdef DEBUG_DAC
	usart_write_line(USART1, "Enable DAC\n");
#endif

	// Register SSC1 to interrupt controller
	INTC_register_interrupt(&pcm1794a_int_handler, AVR32_SSC1_IRQ, AVR32_INTC_INT2);

	// Receive clock mode register
	pcm1794a_ssc_pio->tcmr = AVR32_SSC_TCMR_CKS_TK_PIN	// Transmit clock = TX_PIN
			   | (0 << AVR32_SSC_TCMR_CKO_OFFSET) 		// Clock only during data transfers
			   | (5 << AVR32_SSC_TCMR_START_OFFSET);	// Receive start when detecting rising edge on TF

	// Receive frame mode register
	pcm1794a_ssc_pio->tfmr = (31 << AVR32_SSC_TFMR_DATLEN_OFFSET)	// 32 bit data length
			   | AVR32_SSC_TFMR_MSBF_MASK				// MSB first
			   | (1 << AVR32_SSC_TFMR_DATNB_OFFSET)		// Two data packets per frame sync
			   | (0 << AVR32_SSC_TFMR_FSLEN_OFFSET)
			   | (0 << AVR32_SSC_TFMR_FSOS_OFFSET);		// None

	// PIO enable
	pcm1794a_pio->per  |= 1 << PCM1794A_MONO_PIN
			  | 1 << PCM1794A_CHSL_PIN
			  | 1 << PCM1794A_DEM_PIN
			  | 1 << PCM1794A_MUTE_PIN
			  | 1 << PCM1794A_FMT0_PIN
			  | 1 << PCM1794A_FMT1_PIN
			  | 1 << PCM1794A_ZERO_PIN
			  | 1 << PCM1794A_RST_PIN;

	// Pull-up disable
	pcm1794a_pio->pudr |= 1 << PCM1794A_MONO_PIN
			  | 1 << PCM1794A_CHSL_PIN
			  | 1 << PCM1794A_DEM_PIN
			  | 1 << PCM1794A_MUTE_PIN
			  | 1 << PCM1794A_FMT0_PIN
			  | 1 << PCM1794A_FMT1_PIN
			  | 1 << PCM1794A_RST_PIN
			  | 1 << PCM1794A_ZERO_PIN;

	// Output enable
	pcm1794a_pio->oer  |= 1 << PCM1794A_MONO_PIN
			  | 1 << PCM1794A_CHSL_PIN
			  | 1 << PCM1794A_DEM_PIN
			  | 1 << PCM1794A_MUTE_PIN
			  | 1 << PCM1794A_FMT0_PIN
			  | 1 << PCM1794A_FMT1_PIN
			  | 1 << PCM1794A_RST_PIN;

	// Output write enable
	pcm1794a_pio->ower |= 1 << PCM1794A_MONO_PIN
			  | 1 << PCM1794A_CHSL_PIN
			  | 1 << PCM1794A_DEM_PIN
			  | 1 << PCM1794A_MUTE_PIN
			  | 1 << PCM1794A_FMT0_PIN
			  | 1 << PCM1794A_FMT1_PIN
			  | 1 << PCM1794A_RST_PIN;

	// Input glitch filter enable
	pcm1794a_pio->ifer |= 1 << PCM1794A_ZERO_PIN;
}

void pcm1794a_write(void){
	// Transfer settings to outputs
	pcm1794a_pio->odsr = ( (pcm1794a_pio->odsr & ~(1 << PCM1794A_MONO_PIN)) | (pcm1794a.mono << PCM1794A_MONO_PIN) );
	pcm1794a_pio->odsr = ( (pcm1794a_pio->odsr & ~(1 << PCM1794A_CHSL_PIN)) | (pcm1794a.chsl << PCM1794A_CHSL_PIN) );
	pcm1794a_pio->odsr = ( (pcm1794a_pio->odsr & ~(1 << PCM1794A_DEM_PIN))  | (pcm1794a.dem  << PCM1794A_DEM_PIN)  );
	pcm1794a_pio->odsr = ( (pcm1794a_pio->odsr & ~(1 << PCM1794A_MUTE_PIN)) | (pcm1794a.mute << PCM1794A_MUTE_PIN) );
	pcm1794a_pio->odsr = ( (pcm1794a_pio->odsr & ~(1 << PCM1794A_RST_PIN))  | (pcm1794a.rst  << PCM1794A_RST_PIN)  );
	pcm1794a_pio->odsr = ( (pcm1794a_pio->odsr & ~(1 << PCM1794A_FMT0_PIN)) | (pcm1794a.fmt0 << PCM1794A_FMT0_PIN) );
	pcm1794a_pio->odsr = ( (pcm1794a_pio->odsr & ~(1 << PCM1794A_FMT1_PIN)) | (pcm1794a.fmt1 << PCM1794A_FMT1_PIN) );
}


__attribute__((__interrupt__)) void pcm1794a_int_handler(void){

#ifdef DEBUG_DAC
	usart_write_line(USART1, "pcm1794 irq\n");
#endif

	// Read interrupt status and clear pending interrupts
	unsigned int irq_status = 	pcm1794a_ssc_pio->sr & 	pcm1794a_ssc_pio->imr;

		// Transmit ready?
		if ((irq_status & AVR32_SSC_IER_TXRDY_MASK) != 0){

			// Pointer to next sample in the audio buffer
			signed short int * buffer = (signed short int*)&aud_buf[dac_buf_indx][0];

				// Number of samples in buffer
				unsigned int size = aud_buf_sizes[dac_buf_indx];

				// Buffer ready
				if(aud_buf_rdy > 0){

					if(dac_samples_indx < size){
					    // Still within buffer limits, read samples
						pcm1794a_ssc_pio->thr = ((*(buffer + dac_samples_indx) << 8 ) & 0xff00) | ((*(buffer + dac_samples_indx)  >> 8 ) & 0xff);
						dac_samples_indx++;

						if(dac_samples_indx >= size){
							// Buffer empty, switch buffer.
							dac_samples_indx = 0;
							dac_buf_indx = (dac_buf_indx + 1) % AUDIO_BUFFER_COUNT;
							aud_buf_rdy--;
						}
					}else{
						// Switch buffer and read samples
						dac_samples_indx = 0;
						dac_buf_indx = (dac_buf_indx + 1 ) % AUDIO_BUFFER_COUNT;
						aud_buf_rdy--;
						pcm1794a_ssc_pio->thr = ((*(buffer + dac_samples_indx) << 8 ) & 0xff00) | ((*(buffer + dac_samples_indx)  >> 8 ) & 0xff);
						dac_samples_indx++;
					}
				}else{
					// If no audio data available, output zero
					pcm1794a_ssc_pio->thr = 0x0000;
				}
		}
}

void pcm1794a_spdif_mode(void){

	// Assign io pins to pio controller to avoid drive conflict
	pcm1794a_pio->per |= 1 << AVR32_SSC1_TX_DATA_0_PIN
					  | 1 << AVR32_SSC1_TX_CLOCK_0_PIN
					  | 1 << AVR32_SSC1_TX_FRAME_SYNC_0_PIN;

	// Disable outputs
	pcm1794a_pio->odr |= 1 << AVR32_SSC1_TX_DATA_0_PIN
				      | 1 << AVR32_SSC1_TX_CLOCK_0_PIN
				      | 1 << AVR32_SSC1_TX_FRAME_SYNC_0_PIN;

	// Disable ssc
	pcm1794a_ssc_pio->idr = AVR32_SSC_IER_TXRDY_MASK;
	pcm1794a_ssc_pio->cr &= ~(1 << AVR32_SSC_CR_TXEN_OFFSET);
}

void pcm1794a_usb_mode(void){

	// Assign io pins to SSC
	gpio_enable_module(PCM1794A_GPIO_MAP, sizeof(PCM1794A_GPIO_MAP) / sizeof(PCM1794A_GPIO_MAP[0]));

	// Enable ssc
	pcm1794a_ssc_pio->ier = AVR32_SSC_IER_TXRDY_MASK;
	pcm1794a_ssc_pio->cr |= 1 << AVR32_SSC_CR_TXEN_OFFSET;
}

int pcm1794a_probe_zero(void){
	return (pcm1794a_pio->odsr & (1 << PCM1794A_ZERO_PIN));
}

void pcm1794a_set_mute(void){
	pcm1794a.mute = 1;
	pcm1794a_pio->odsr |= 1 << PCM1794A_MUTE_PIN;
}

void pcm1794a_clr_mute(void){
	pcm1794a.mute = 0;
	pcm1794a_pio->odsr &= ~(1 << PCM1794A_MUTE_PIN);
}

void pcm1794a_set_dem(void){
	pcm1794a.dem = 1;
	pcm1794a_pio->odsr |= 1 << PCM1794A_DEM_PIN;
}

void pcm1794a_clr_dem(void){
	pcm1794a.dem = 0;
	pcm1794a_pio->odsr &= ~(1 << PCM1794A_MUTE_PIN);
}

void pcm1794a_set_rst(void){
	pcm1794a.rst = 1;
	pcm1794a_pio->odsr &= ~(1 << PCM1794A_RST_PIN);
}
void pcm1794a_clr_rst(void){
	pcm1794a.rst = 0;
	pcm1794a_pio->odsr |= 1 << PCM1794A_RST_PIN;
}
void pcm1794_set_format(int format){

	switch(format){
	case RIGHT_JUSTIFIED_16_BIT:
		pcm1794a.mono = 0;
		pcm1794a.chsl = 0;
		pcm1794a.fmt1 = 1;
		pcm1794a.fmt0 = 0;
		break;
	case RIGHT_JUSTIFIED_24_BIT:
		pcm1794a.mono = 0;
		pcm1794a.chsl = 0;
		pcm1794a.fmt1 = 1;
		pcm1794a.fmt0 = 1;
		break;
	case LEFT_JUSTIFIED_24_BIT:
		pcm1794a.mono = 0;
		pcm1794a.chsl = 0;
		pcm1794a.fmt1 = 0;
		pcm1794a.fmt0 = 1;
		break;
	case I2S_24_BIT:
		pcm1794a.mono = 0;
		pcm1794a.chsl = 0;
		pcm1794a.fmt1 = 0;
		pcm1794a.fmt0 = 0;
		break;
	default:
		pcm1794a.mono = 0;
		pcm1794a.chsl = 0;
		pcm1794a.fmt1 = 1;
		pcm1794a.fmt0 = 0;
	}
	// Output new settings
	pcm1794a_write();

}
