/******************************************************************************
 * pcm1794a.h
 *
 *	\brief This file defines functions to control the pcm1794a.
 *
 *	PCM1794a is the dataconverter implemented in the system. This receives data
 *	from either the dir9001 SPDIF receiver or from the USB connection via the
 *	SSC1 module. Clocking is performed from the dir9001 in SPDIF mode and from
 *	the cdce706 external clock in USB mode. Since both the dir9001 and the
 *	SSC1 module/clock is connected to the same clock and data lines, they cannot
 *	be active simulaneously. When the system is in SPDIF mode the SSC1 must be
 *	deactivated to avoid drive conflict. In USB mode is audio data clocked from
 *	SSC1 to the DAC by the external clock and the signal from dir9001 is
 *	tristated. The control pins of the DAC is driven by general purpose io pins.
 *
 *  Created on: Feb 27, 2009
 *      Author: Espen Karlsen
 ******************************************************************************/

#ifndef PCM1794A_H_
#define PCM1794A_H_

#include <avr32/io.h>
#include "doxaUSB.h"

// DAC settings
typedef struct pcm1794a_s {
	unsigned int mute : 1;
	unsigned int fmt0 : 1;
	unsigned int fmt1 : 1;
	unsigned int mono : 1;
	unsigned int chsl : 1;
	unsigned int dem  : 1;
	unsigned int rst  : 1;
	unsigned int zero : 1;
} pcm1794a_s;

// Map ssc1 to io pins
static const gpio_map_t PCM1794A_GPIO_MAP =
  {
    {AVR32_SSC1_TX_DATA_0_PIN, AVR32_SSC1_TX_DATA_0_FUNCTION},
    {AVR32_SSC1_TX_CLOCK_0_PIN, AVR32_SSC1_TX_CLOCK_0_FUNCTION},
    {AVR32_SSC1_TX_FRAME_SYNC_0_PIN, AVR32_SSC1_TX_FRAME_SYNC_0_FUNCTION}
  };

/*! \brief Initialize pcm1794a dac
 *
 * Setup SSC1 as bus master
 * Register SSC1 interrupt routine
 * Setup pio pins for control signals
 */
void pcm1794a_init(void);

/*! \brief Output dac control settings
 *
 * Sets pio control pins according to the present dac settings in
 * the pcm1794a_s struct.
 */
void pcm1794a_write(void);

/*! \brief Interrupt routine for SSC1
 *
 * Called every time a SSC1 is ready to send data to the DAC
 */
void pcm1794a_int_handler(void);

/*! \brief Enter SPDIF mode
 *
 * Assign SSC1 pins to pio controller and disable outputs.
 * Disable SSC1 and deactivate its interrupt
 */
void pcm1794a_spdif_mode(void);

/*! \brief Enter USB mode
 *
 * Assign pio pins to SSC1 module and enable SSC1 and its interrupt.
 */
void pcm1794a_usb_mode(void);


/*! \brief Mute DAC
 *
 */
void pcm1794a_set_mute(void);


/*! \brief Unmute DAC
 *
 */
void pcm1794a_clr_mute(void);

/*! \brief Set deemphasis on
 *
 */
void pcm1794a_set_dem(void);

/*! \brief Set deemphasis off
 *
 */
void pcm1794a_clr_dem(void);

/*! \brief Activate DAC reset
 *
 */
void pcm1794a_set_rst(void);

/*! \brief Deactivate DAC reset
 *
 */
void pcm1794a_clr_rst(void);

/*! \brief Set DAC data format
 *  \param format
 *  	\arg RIGHT_JUSTIFIED_16_BIT
 *  	\arg RIGHT_JUSTIFIED_24_BIT
 *  	\arg LEFT_JUSTIFIED_24_BIT
 *  	\arg I2S_24_BIT
 */
void pcm1794_set_format(int format);

/*! \brief Read DAC zero pin
 *
 * \return ZERO pin value
 * 		\arg 0 if zero
 * 		\arg (1 << PCM1794A_ZERO_PIN) if set
 */
int pcm1794a_probe_zero(void);

#endif /* PCM1794A_H_ */
