/******************************************************************************
 * ui.c
 *
 * \brief This file defines functions to control the buttons and LEDs.
 *
 *	Four buttons are connected to the PIOB module to control volume up, volume
 *	down, mute and source select. Three leds are connected to the same module
 *	to indicate mute status and source selection.
 *
 *  Created on: May 24, 2009
 *      Author: Espen Karlsen
 ******************************************************************************/

#include "ui.h"
#include <avr32/io.h>
#include "doxaUSB.h"
#include "intc.h"
#include "dir9001.h"
#include "pcm1794a.h"

void init_buttons(){

	volatile avr32_pio_t *piob = &AVR32_PIOB;
	volatile avr32_rtc_t *rtc = &AVR32_RTC;

	// Pio enable
	piob->per 	|= (1 << VOLUME_UP_BUTTON_PIN)
				| (1 << VOLUME_DOWN_BUTTON_PIN)
				| (1 << SOURCE_CHANGE_BUTTON_PIN)
				| (1 << SOURCE_USB_LED_PIN)
				| (1 << SOURCE_SPDIF_LED_PIN)
				| (1 << VOLUME_MUTE_LED_PIN)
				| (1 << VOLUME_MUTE_BUTTON_PIN);

	// Pull up enable
	piob->puer 	|= (1 << VOLUME_UP_BUTTON_PIN)
				| (1 << VOLUME_DOWN_BUTTON_PIN)
				| (1 << VOLUME_MUTE_BUTTON_PIN)
				| (1 << SOURCE_CHANGE_BUTTON_PIN);

	// Pull up disable
	piob->pudr 	|= (1 << SOURCE_USB_LED_PIN)
				| (1 << SOURCE_SPDIF_LED_PIN)
				| (1 << VOLUME_MUTE_LED_PIN);

	// Output enable
	piob->oer  	|= (1 << SOURCE_USB_LED_PIN)
				| (1 << SOURCE_SPDIF_LED_PIN)
				| (1 << VOLUME_MUTE_LED_PIN);

	// Output disable
	piob->odr	|= (1 << VOLUME_UP_BUTTON_PIN)
				| (1 << VOLUME_DOWN_BUTTON_PIN)
				| (1 << VOLUME_MUTE_BUTTON_PIN)
				| (1 << SOURCE_CHANGE_BUTTON_PIN);

	// Output write enable
	piob->ower 	|= (1 << SOURCE_USB_LED_PIN)
				| (1 << SOURCE_SPDIF_LED_PIN)
				| (1 << VOLUME_MUTE_LED_PIN);

	// Output write disable
	piob->owdr 	|= (1 << VOLUME_UP_BUTTON_PIN)
				| (1 << VOLUME_DOWN_BUTTON_PIN)
				| (1 << VOLUME_MUTE_BUTTON_PIN)
				| (1 << SOURCE_CHANGE_BUTTON_PIN);


	// Glitch input filter enable
	piob->ifer	|= (1 << VOLUME_UP_BUTTON_PIN)
				| (1 << VOLUME_DOWN_BUTTON_PIN)
				| (1 << VOLUME_MUTE_BUTTON_PIN)
				| (1 << SOURCE_CHANGE_BUTTON_PIN);

	// Clear pending interrupts
	piob->idr |= piob->isr;

	// Interrupt enable
	piob->ier	|= (1 << VOLUME_UP_BUTTON_PIN)
				| (1 << VOLUME_DOWN_BUTTON_PIN)
				| (1 << VOLUME_MUTE_BUTTON_PIN)
				| (1 << SOURCE_CHANGE_BUTTON_PIN);


	// Register buttons to interrupt controller, priority level 1
	INTC_register_interrupt(&buttons_int_handler, AVR32_PIOB_IRQ, AVR32_INTC_INT1);

	// Update LED settings
	leds_update();

	// Initialize Real Time Counter

	// Flip over at this value
	rtc->top = 4000;
	// Enable counter and flip over at rtc->top
	rtc->ctrl = AVR32_RTC_EN_MASK | AVR32_RTC_TOPEN_MASK;
	// Enable interrupt when flipping over
	rtc->ier = AVR32_RTC_TOPI_MASK;

	// Register real time counter interrupt routine, priority level 2
	INTC_register_interrupt(&rtc_int_handler, AVR32_RTC_IRQ, AVR32_INTC_INT2);
}

void leds_update(){

	volatile avr32_pio_t *piob = &AVR32_PIOB;

	// Control source LEDs
	switch(get_source()){
		case USB:
			piob->odsr |= (1 << SOURCE_USB_LED_PIN);
			piob->odsr &= ~(1 << SOURCE_SPDIF_LED_PIN);
			break;
		case SPDIF:
			piob->odsr |= (1 << SOURCE_SPDIF_LED_PIN);
			piob->odsr &= ~(1 << SOURCE_USB_LED_PIN);
			break;
		case AUX:
			piob->odsr &= ~(1 << SOURCE_USB_LED_PIN);
			piob->odsr &= ~(1 << SOURCE_SPDIF_LED_PIN);
			break;
	}

	// Control mute LED
	switch(get_mute()){
		case MUTE_OFF:
			piob->odsr &= ~(1 << VOLUME_MUTE_LED_PIN);
			break;
		case MUTE_ON:
			piob->odsr |= (1 << VOLUME_MUTE_LED_PIN);
			break;
	}
}


__attribute__((__interrupt__)) void buttons_int_handler(void){

	// Memory pointers
	volatile avr32_pio_t *piob = &AVR32_PIOB;
	volatile avr32_rtc_t *rtc = &AVR32_RTC;

	// Read interrupt status and clear pending interrupts
	int irq_status = piob->isr;

	// Interrupt disable until new settings are shifted out by SPI
	piob->idr	|= (1 << VOLUME_UP_BUTTON_PIN)
				| (1 << VOLUME_DOWN_BUTTON_PIN)
				| (1 << VOLUME_MUTE_BUTTON_PIN)
				| (1 << SOURCE_CHANGE_BUTTON_PIN);

	// Reset real time counter
	rtc->val = 0;

	// Check which pin that was pressed

	// Volume up button pressed
	if(irq_status & (1 << VOLUME_UP_BUTTON_PIN) && ~(piob->odsr & (1 << VOLUME_UP_BUTTON_PIN)) ){
		set_volume(LEFT_VOL, get_volume(LEFT_VOL)+1);
		set_volume(RIGHT_VOL, get_volume(RIGHT_VOL)+1);
	}

	// Volume down button pressed
	if(irq_status & (1 << VOLUME_DOWN_BUTTON_PIN) && ~(piob->odsr & (1 << VOLUME_DOWN_BUTTON_PIN)) ){

		set_volume(LEFT_VOL, get_volume(LEFT_VOL)-1);
		set_volume(RIGHT_VOL, get_volume(RIGHT_VOL)-1);
	}

	// Source change button pressed
	if(irq_status & (1 << SOURCE_CHANGE_BUTTON_PIN) && ~(piob->odsr & (1 << SOURCE_CHANGE_BUTTON_PIN)) ){

		switch(get_source()){
		case USB:
			set_source(SPDIF);
			piob->odsr |= (1 << SOURCE_SPDIF_LED_PIN);
			piob->odsr &= ~(1 << SOURCE_USB_LED_PIN);
			pcm1794a_spdif_mode();
			dir9001_enable();
			break;
		case SPDIF:
			set_source(AUX);
			piob->odsr &= ~(1 << SOURCE_USB_LED_PIN);
			piob->odsr &= ~(1 << SOURCE_SPDIF_LED_PIN);
			dir9001_disable();
			break;
		case AUX:
			set_source(USB);
			piob->odsr |= (1 << SOURCE_USB_LED_PIN);
			piob->odsr &= ~(1 << SOURCE_SPDIF_LED_PIN);
			pcm1794a_usb_mode();
			break;
		default:
			break;
		}

	}

	// Mute button pressed
	if(irq_status & (1 << VOLUME_MUTE_BUTTON_PIN) && ~(piob->odsr & (1 << VOLUME_MUTE_BUTTON_PIN))){

		if(get_mute()){
			set_mute(MUTE_OFF);
			piob->odsr &= ~(1 << VOLUME_MUTE_LED_PIN);
		}
		else{
			set_mute(MUTE_ON);
			piob->odsr |= (1 << VOLUME_MUTE_LED_PIN);
		}
	}

	// Output new settings to volume control
	write_volume_ctrl();

}

// RTC interrupt routine
__attribute__((__interrupt__)) void rtc_int_handler(void){

	// Memory pointers
	volatile avr32_pio_t *piob = &AVR32_PIOB;
	volatile avr32_rtc_t *rtc = &AVR32_RTC;

	// Clear pending interrupts
	piob->idr |= piob->isr;

	// Enable button interrupt
	piob->ier	= (1 << VOLUME_UP_BUTTON_PIN)
				| (1 << VOLUME_DOWN_BUTTON_PIN)
				| (1 << VOLUME_MUTE_BUTTON_PIN)
				| (1 << SOURCE_CHANGE_BUTTON_PIN);

	// Clear RTC interrupt
	rtc->icr = AVR32_RTC_TOPI_MASK;

	return;
}
