/******************************************************************************
 * volumecontrol.c
 *
 * \brief This file defines functions to control the volume control implemented.
 *
 * The volume control consists of six relays for volume control, and one
 * relay for controlling audio source and one for controlling mute. Two
 * volume channels are implemented. The relays are controlled by two 8 bit
 * shift registers connected to the spi 1 module. The data and clock pin
 * are controlled through this module while the latch pin is controlled by
 * general io. When the latch pin goes from low to high, the shift
 * register content are transferred to the output.
 *
 *  Created on: May 24, 2009
 *      Author: Espen Karlsen
 ******************************************************************************/

#include "volumecontrol.h"
#include "gpio.h"
#include "doxaUSB.h"
#include "compiler.h"
#include "intc.h"
#include "usart.h"
#include <avr32/io.h>


// Memory pointer to the spi 1 module
volatile avr32_spi_t *spi1 = &AVR32_SPI1;

// Volume control settings struct
volume_ctrl_t volume_ctrl;


void init_volume_ctrl(void){

	// Memory pointer
	volatile avr32_pio_t *pioa = &AVR32_PIOA;

	// Setup latch clock pio
	pioa->per |= (1 << VOLUME_LATCH_PIN);	// Pio enable
	pioa->pudr |= (1 << VOLUME_LATCH_PIN);	// Pull up disable
	pioa->oer  |= (1 << VOLUME_LATCH_PIN);	// Output enable
	pioa->ower |= (1 << VOLUME_LATCH_PIN);	// Output write enable
	pioa->odsr |= (1 << VOLUME_LATCH_PIN);	// Output data

	// Assign io pins to spi module
	gpio_enable_module(VOLUME_GPIO_MAP, sizeof(VOLUME_GPIO_MAP) / sizeof(VOLUME_GPIO_MAP[0]));

	// SPI master mode
	spi1->mr = AVR32_SPI_MSTR_MASK;

	// Setup SPI 1, chipselect 0
	spi1->csr0 = AVR32_SPI_CSR0_NCPHA_MASK
			   | AVR32_SPI_CSR0_CSAAT_MASK 									// Chip select remains active after a transfer
			   | AVR32_SPI_CSR0_BITS_16_BPT << AVR32_SPI_CSR0_BITS_OFFSET	// 16 bit per transfer
			   | 10<< AVR32_SPI_CSR0_SCBR_OFFSET; 							// Baud rate = PBA freq / 10

	// Register spi 1 to interrupt controller
	INTC_register_interrupt(&volume_ctrl_int_handler, AVR32_SPI1_IRQ, AVR32_INTC_INT2);

	// Shift out volume control settings
	write_volume_ctrl();

}


int write_volume_ctrl(void){

	// Memory pointer
	volatile avr32_pio_t *pioa = &AVR32_PIOA;

	// If AUX is choosen, the source selection relay should enter
	// the on state, otherwise it should be off.
	int source;
	volume_ctrl.source == AUX ? (source = 1) : (source = 0);

	// Arrange bits in accordance with the hardware configuration
	unsigned short setting = source << 0
							| source << 8
							| volume_ctrl.mute << 15
							| volume_ctrl.mute << 7
							| volume_ctrl.volume_l << 1
							| volume_ctrl.volume_r << 9;

	spi1->cr = AVR32_SPI_CR_SPIEN_MASK;			// Enable spi

	// If transmit register is empty, send data
	if(spi1->sr & AVR32_SPI_TXEMPTY_MASK){
		pioa->odsr &= ~(1 << VOLUME_LATCH_PIN);	// SREG latch clock low
		spi1->tdr = setting;					// Write settins to transmit data register
		spi1->ier = AVR32_SPI_IER_TXEMPTY_MASK;	// Enable spi txempty interrupt
		return 0;	// Success
	}else{
		return -1;	// Transmit register not empty
	}
}


__attribute__((__interrupt__)) void volume_ctrl_int_handler(void){

	// Memory pointer
	volatile avr32_pio_t *pioa = &AVR32_PIOA;

	// If SPI transmission is completed
	if(spi1->sr & AVR32_SPI_TXEMPTY_MASK){
		spi1->idr |= AVR32_SPI_IER_TXEMPTY_MASK;	// Disable interrupt
		spi1->cr = AVR32_SPI_CR_SPIDIS_MASK;		// Disable spi module
		pioa->odsr |= 1 << VOLUME_LATCH_PIN;		// Latch SREG content to outputs
	}
}


signed char get_volume(enum vol_channel ch){
	switch(ch){
		case LEFT_VOL: 	return volume_ctrl.volume_l;
		case RIGHT_VOL: return volume_ctrl.volume_r;
		default: 		return -1;
	}
}

signed char set_volume(enum vol_channel ch, signed char value){
	switch(ch){
		case LEFT_VOL:
			if((0 <=  value) && (value <= VOLUME_MAX))
				return volume_ctrl.volume_l = value;
			break;
		case RIGHT_VOL:
			if((0 <=  value) && (value <= VOLUME_MAX))
				return volume_ctrl.volume_r = value;
			break;
		default:
			break;
		}
	return -1;
}


void set_mute(enum mute_sts sts){
	switch(sts){
		case MUTE_ON:
			volume_ctrl.mute = MUTE_ON;
			break;
		case MUTE_OFF:
			volume_ctrl.mute = MUTE_OFF;
			break;
		default:
			break;
	}
}

enum mute_sts get_mute(void){
	return volume_ctrl.mute;
}

void set_source(enum source src){
	volume_ctrl.source = src;
}

enum source get_source(void){
	return volume_ctrl.source;
}
