/******************************************************************************
 * volumecontrol.c
 *
 * \brief This file defines functions to control the volume control.
 *
 * The volume control consists of six relays for volume control, and one
 * relay for controlling audio source and one for controlling mute. Two
 * volume channels are implemented. The relays are controlled by two 8 bit
 * shift registers connected to the spi 1 module. The data and clock pin
 * are controlled through this module while the latch pin is controlled by
 * general io. When the latch pin goes from low to high, the shift
 * register content are transferred to the output.
 *
 *  Created on: May 24, 2009
 *      Author: Espen Karlsen
 ******************************************************************************/


#ifndef VOLUMECONTROL_H_
#define VOLUMECONTROL_H_

#include "gpio.h"

// Max volume 6 bit -> 64 levels
#define VOLUME_MAX 63
#define SENDING 	0
#define IDLE		1

// Possible channels with volume control
enum vol_channel{
	LEFT_VOL,
	RIGHT_VOL
};

// Mute status
enum mute_sts{
	MUTE_OFF,
	MUTE_ON
};

// Possible source selections
enum source{
  USB,
  SPDIF,
  AUX
};

/*! \brief Volume control settings
 *
 */
typedef struct volume_ctrl_s{
	// Left volume
	unsigned char volume_l;
	// Right volume
	unsigned char volume_r;
	// Mute status
	unsigned char mute;
	// Input audio source
	enum source source;
} volume_ctrl_t;


/* IO mapping of SPI 1 module to the output pins.
 * Only Master output slave input (MOSI) and clock pins are used.
*/
static const gpio_map_t VOLUME_GPIO_MAP =
  {
	{AVR32_SPI1_MOSI_0_PIN, AVR32_SPI1_MOSI_0_FUNCTION},
	{AVR32_SPI1_SCK_0_PIN, AVR32_SPI1_SCK_0_FUNCTION},
  };

/* Initializes the volume control
*/

/*! \brief Initialize volume control hardware
 *
 * Sets up the pio controler for the LATCH pin to the SREGs and SPI module.
 * Sets up the spi module and register the spi irq.
 * Shifts volume control settings to output.
 */
void init_volume_ctrl(void);

/*! \brief Shifts the volume control settings to the output
 * \return
 * 		\retval 0 Success
 * 		\retval -1 SPI transmit register not empty
 */
int write_volume_ctrl(void);

/*! \brief Volume control interrupt routine
 *
 * Called when spi transmission is completed.
 * Disables spi module and spi interrupts.
 * Pulls the latch pin high to transfer SREG settings to their outputs.
 */
void volume_ctrl_int_handler(void);

/*! \brief Get volume value
 *
 * \param ch Channel to get the volume of
 * \return volume value of ch
 */
signed char get_volume(enum vol_channel ch);

/*! \brief Set volume on specified channel
 *
 * \param ch Channel to set volume
 * \param value Volume value
 * \return New volume value on ch
 */
signed char set_volume(enum vol_channel ch, signed char value);

/*! \brief Set mute on all channels
 *
 * \param mute_sts
 * 		\arg MUTE_ON
 * 		\arg MUTE_OFF
 */
void set_mute(enum mute_sts);

/*! \brief Get mute status of all channels
 *
 * \return mute_sts
 * 		\arg MUTE_ON
 * 		\arg MUTE_OFF
 */
enum mute_sts get_mute(void);

/*! \brief Select input audio source
 *
 * \param src
 * 		\arg USB
 * 		\arg SPDIF
 * 		\arg AUX
 */
void set_source(enum source src);

/*! \brief Get input audio source
 *
 * \return source
 * 		\arg USB
 * 		\arg SPDIF
 * 		\arg AUX
 */
enum source get_source(void);

#endif /* VOLUMECONTROL_H_ */
